"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Themes helper class.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt
from PySide6.QtSvg import QSvgRenderer
from PySide6.QtGui import QColor, QIcon, QPixmap, QPainter

import os
import logging

from typing import Union

from ..settings import Settings
from ..theme import Theme
from ..enums.themes import Themes


class ThemeHelper:
    """
    This class is intended to help with theme assets (colors, icons, etc.)
    """

    def __init__(self):
        super().__init__()

        self.logger = logging.getLogger('theme_helper')

        self.logger.debug('Theme helper is activated')

        self.settings = Settings()

        theme_name = self.settings.app_theme
        # Check the theme is still in the themes list with fallback to the default one
        if theme_name not in [_theme.name.lower() for _theme in Themes]:  # Or: list(Themes.__members__.keys()):
            theme_name = str(Themes.default())

        self.theme = Theme(theme=theme_name)

    def get_app_icon_path(self):
        return self.theme.get_assets_dir('notolog-logo.png')

    def get_color(self, color_name: str, css_format: bool = False) -> Union[int, str, None]:
        """
        Get a particular theme color by its name.
        @param color_name: string the color name
        @param css_format: bool whether to convert to CSS format or not. Works with hex colors.
        @return: the color value
        """

        """
        It's possible to pass css_format param but all colors will be converted in this case.
        It can be too resource greedy if cache is in use.

        The colors have already been converted to int(color, 16) format.
        """
        theme_colors = self.theme.get_colors()
        if not theme_colors:
            return None
        color = theme_colors.get(color_name)\
            if color_name in theme_colors else None  # theme_colors.get('color_default')

        if color is None:
            self.logger.debug(f"Skip not set color '{color_name}', theme '{self.theme.theme}'")
            return None

        if isinstance(color, int):
            # Hex color format
            if color and css_format:
                # Convert to CSS format (#RRGGBB)
                return "#{:06x}".format(color).upper()
            return color

        # String color format
        return color

    def get_css(self, css_name=str) -> Union[str, None]:
        """
        Get CSS file content by its name.
        @param css_name: the name
        @return: string of the CSS file content
        """
        theme_css = self.theme.get_css()
        css_file = theme_css.get(css_name) if theme_css else None
        if css_file and os.path.isfile(css_file):
            # Load CSS-file and return its content
            return self.theme.load_css(css_file)
        return None

    def get_icon(self, theme_icon: str, system_icon: str = None, color: QColor = None,
                 # Can be adjusted independently
                 width: int = 64, height: int = 64) -> QIcon:
        """
        Get QIcon by params

        Args:
            theme_icon (str): Theme icon file name
            system_icon (str, optional): System icon name as a fallback
            color (QColor, optional): Color of the SVG icon
            width (int, optional): Icon width
            height (int, optional): Icon height

        Returns:
            QIcon: Either from file or from the system theme
        """
        theme_icon_dir = os.path.join(self.theme.get_theme_dir(), 'icons')
        if not os.path.exists(theme_icon_dir):
            # Get default icons then
            theme_icon_dir = os.path.join(self.theme.get_default_theme_dir(), 'icons')
        theme_icon_path = os.path.join(theme_icon_dir, theme_icon)
        if os.path.isfile(theme_icon_path):
            if not color:
                default_icon_color = self.get_color('toolbar_icon_color_default')
                color = QColor(default_icon_color)
                self.logger.debug('Applying default color fallback "#%x"' % default_icon_color)
            return QIcon(self.svg_to_pixmap(svg_filepath=theme_icon_path, color=color, width=width, height=height))
        else:
            # With a fallback: QIcon.fromTheme(system_icon, QIcon(theme_icon))
            return QIcon.fromTheme(system_icon)

    def svg_to_pixmap(self, svg_filepath: str, color: QColor, width: int = 64, height: int = 64) -> QPixmap:
        """
        Helps to load an SVG file and convert it to QPixmap

        Args:
            svg_filepath (str): SVG file path
            color (QColor): Color of the SVG icon
            width (int, optional): Width of the icon. Defaults to 64.
            height (int, optional): Height of the icon. Defaults to 64.

        Returns:
            QPixmap: Pixmap data
        """
        renderer = QSvgRenderer(svg_filepath)
        pixmap = QPixmap(width, height)
        pixmap.fill(Qt.GlobalColor.transparent)
        painter = QPainter(pixmap)
        # Render destination image
        renderer.render(painter)
        """
        The output is the source, where the alpha is reduced by that of the destination

        More about QPainter.CompositionMode
        * https://doc.qt.io/qt-6/qpainter.html#CompositionMode-enum
        """
        painter.setCompositionMode(QPainter.CompositionMode.CompositionMode_SourceIn)
        # Fill source image with color
        painter.fillRect(pixmap.rect(), color)
        painter.end()
        return pixmap



# Latin lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "API OpenAI",

    "module_openai_api_label": "API OpenAI",
    "module_openai_api_url_label": "URL API",
    "module_openai_api_url_input_placeholder_text": "URL API",
    "module_openai_api_url_input_accessible_description":
        "URL OpenAI API est locus terminatio API, qui variare potest secundum officium et versionem.\n"
        "Auxiliarius AI hoc utitur pro functione colloquii aut textus complementis.\n"
        "Ad conspectum hodiernum URL, documenta officialia OpenAI API consulenda sunt.",
    "module_openai_api_key_label": "Clavis API",
    "module_openai_api_key_input_placeholder_text": "Clavis API",
    "module_openai_api_key_input_accessible_description":
        "Clavis API OpenAI est secretum symbolum ad validationem petitionum apud terminationem API.",
    "module_openai_api_supported_models_label": "Modellos sustentata",
    "module_openai_api_model_names_combo_placeholder_text": "Elige Modello",
    "module_openai_api_model_names_combo_accessible_description":
        "Elige inter modellos sustentata pro colloquiis chattationis.",

    "module_openai_api_base_system_prompt_label": "Promptus Systematis",
    "module_openai_api_base_system_prompt_edit_placeholder_text":
        "Promptus systematis basis quae praecedit omnem postulationem",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "Promptus systematis basis quae praecedit omnem postulationem.\n"
        "Plurimum est textus simplex cum praeceptis vel characteribus muneri.",

    "module_openai_api_base_response_temperature_label": "Temperatura: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "Varietatem responsionum modeli regulat. Valores altiores producunt exitus magis variatos, "
        "dum valores inferiores responsiones magis praedicibiles faciunt.",

    "module_openai_api_base_response_max_tokens_label": "Maximum Responsionum Tokenorum",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "Maximum numerum tokenorum quos in responsione accipere potes, verba et interpunctionem includens, "
        "longitudinem exitus moderans.",

    "module_openai_api_config_prompt_history_size_label": "Magnitudo Historiae Promptorum",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "Numerum introituum in historia promptorum quae systema retinet ad referentiam moderatur.\n"
        "Valor nullos introitus indefinitos permittit."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Clipboard helper class.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtWidgets import QApplication


class ClipboardHelper:
    @staticmethod
    def set_text(text):
        clipboard = QApplication.clipboard()
        clipboard.setText(text)



# Japanese lexemes statusbar.py
lexemes = {
    "statusbar_litter_bin_label": "ごみ箱",
    "statusbar_litter_bin_accessible_name": "ごみ箱の内容を表示",

    "statusbar_save_progress_label": b'\xf0\x9f\x92\xbe',
    "statusbar_encryption_symbol_encrypted_label": b'\xf0\x9f\x94\x92',
    "statusbar_encryption_symbol_unencrypted_label": b'\xf0\x9f\x94\x93',

    "statusbar_mode_label_mode_view": "ビューモード",
    "statusbar_mode_label_mode_edit": "編集モード",
    "statusbar_mode_label_mode_source": "ソースモード",

    "statusbar_source_label_source_markdown": "Markdown",
    "statusbar_source_label_source_html": "HTML",

    "statusbar_encryption_label": "{encryption} {icon}",
    "statusbar_encryption_label_encryption_plain": "プレーン",
    "statusbar_encryption_label_encryption_encrypted": "暗号化",

    "statusbar_cursor_label": "行: {line} → 列: {column}",
    "statusbar_cursor_label_selected": "行: {line} → 列: {column} → 選択: {selected}",
    "statusbar_cursor_label_with_global": "行: {line} → 列: {column} → 位置: {position}",
    "statusbar_cursor_label_selected_with_global": "行: {line} → 列: {column} → 位置: {position} → 選択: {selected}",
}



# Lexica Communia Latine common.py
lexemes = {
    "app_title": "Editorium Notolog",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "Locus filtrationis fasciculorum",

    "menu_action_copy_file_path": "Via fasciculi exemplum",
    "menu_action_rename": "Nomen Mutare",
    "menu_action_delete": "Delere",
    "menu_action_delete_completely": "Delere prorsus",
    "menu_action_restore": "Restituere",
    "menu_action_create_new_dir": "Crea novum directory",

    "dialog_file_rename_title": "Fasciculum renominare",
    "dialog_file_rename_field_label": "Nomen novum fasciculi ingredi",
    "dialog_file_rename_button_ok": "Renominare",
    "dialog_file_rename_warning_exists": "Liber eodem nomine iam existit",

    "dialog_file_delete_title": "Fasciculum delere",
    "dialog_file_delete_text": "Fasciculum \"{file_name}\" delere?",
    "dialog_file_delete_completely_title": "Delere tabulatum prorsus",
    "dialog_file_delete_completely_text": "Delere prorsus tabulatum \"{file_name}\"?",
    "dialog_file_delete_error": "Fasciculum delere non potest, error accidit",
    "dialog_file_delete_error_not_found": "Fasciculus non inventus",

    "dialog_file_restore_title": "Liber Restituere",
    "dialog_file_restore_text": "Liber \"{file_name}\" restituere?",
    "dialog_file_restore_error": "Liber restituere non potest, error accidit",
    "dialog_file_restore_warning_exists": "Liber nomine {file_name} iam existit",

    "dialog_create_new_dir_title": "Crea novum directory",
    "dialog_create_new_dir_label": "Novum nomen directory",
    "dialog_create_new_dir_input_placeholder_text": "Inserere nomen directory",
    "dialog_create_new_dir_button_ok": "Crea",
    "dialog_create_new_dir_button_cancel": "Cancel",
    "dialog_create_new_dir_warning_empty_title": "Novum nomen directory error",
    "dialog_create_new_dir_warning_empty_text": "Nomen directory vacuum esse non potest",
    "dialog_create_new_dir_warning_too_long_title": "Novum nomen directory error",
    "dialog_create_new_dir_warning_too_long_text": "Nomen directory nimis longum est; maxime "
                                                   "{symbols} litterarum permissum est!",
    "dialog_create_new_dir_error_existed": "Directory iam existit",
    "dialog_create_new_dir_error": "Fieri non potest directory creare. Fac ut directory destinatum "
                                   "{base_dir} scribibile sit",

    "dialog_message_box_title": "Nuntius",
    "dialog_message_box_button_ok": "Claudere",

    "action_new_file_first_line_template_text": "Documentum novum",
    "action_open_file_dialog_caption": "Fasciculum aperire",
    "action_save_as_file_dialog_caption": "Fasciculum salvare",

    "dialog_save_empty_file_title": "Serva File Vacuus",
    "dialog_save_empty_file_text": "Permittere servare file sine contento?",

    "dialog_encrypt_file_title": "Fasciculum codicare",
    "dialog_encrypt_file_text": "Fasciculum \"{file_name}\" codicare?",
    "encrypt_file_warning_file_is_already_encrypted": "Fasciculus iam codicatus est!",
    "dialog_encrypt_file_rewrite_existing_title": "Fasciculum exsistentem rescribere",
    "dialog_encrypt_file_rewrite_existing_text": "Fasciculum exsistentem \"{file_path}\" rescribere?",

    "dialog_decrypt_file_title": "Fasciculum decodicare",
    "dialog_decrypt_file_text": "Fasciculum \"{file_name}\" decodicare?",
    "decrypt_file_warning_file_is_not_encrypted": "Fasciculus non est codicatus!",
    "dialog_decrypt_file_rewrite_existing_title": "Fasciculum exsistentem rescribere",
    "dialog_decrypt_file_rewrite_existing_text": "Fasciculum exsistentem \"{file_path}\" rescribere?",

    "dialog_encrypt_new_password_title": "Novum Signum",
    "dialog_encrypt_new_password_label": "Signum:",
    "dialog_encrypt_new_password_input_placeholder_text": "Novum signum ingredere",
    "dialog_encrypt_new_password_hint_label": "Monitum:",
    "dialog_encrypt_new_password_hint_label_description":
        "Monitum non est cryptatum et legi potest ex archivo!"
        "\nNoli uti monitis perspicuis quae facile divinari possunt, ut"
        "\nnatale diem vel similia. Conare uti referentia non facile associata.",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "Ingredere monitum (non necessarium)",
    "dialog_encrypt_new_password_button_ok": "OK",
    "dialog_encrypt_new_password_button_cancel": "Cancellare",
    "dialog_encrypt_new_password_warning_empty_title": "Monitio",
    "dialog_encrypt_new_password_warning_empty_text": "Area signi vacua esse non potest!",
    "dialog_encrypt_new_password_warning_too_long_title": "Monitio",
    "dialog_encrypt_new_password_warning_too_long_text": "Area moniti nimis longa est, maximum {symbols} signa!",

    "dialog_encrypt_password_title": "Ingredere Signum",
    "dialog_encrypt_password_label": "Signum:",
    "dialog_encrypt_password_input_placeholder_text": "Signum ingredere",
    "dialog_encrypt_password_hint_label": "Monitum:",
    "dialog_encrypt_password_button_ok": "OK",
    "dialog_encrypt_password_button_cancel": "Cancellare",

    "dialog_encrypt_password_reset_title": "Signum Cryptatum Reset",
    "dialog_encrypt_password_reset_text": "Certusne es te velle currentem signum cryptatum reset?",
    "dialog_encrypt_password_reset_button_cancel": "Cancellare",
    "dialog_encrypt_password_reset_button_yes": "Ita",

    "dialog_open_link_title": "Vinculum",
    "dialog_open_link_text": "Vinculum \"{url}\" in navigatro aperire?",

    "dialog_reset_settings_title": "Reddere ad initium?",
    "dialog_reset_settings_text":
        "Omnia in configurationibus data delebuntur, et app ad mutationes applicandas restituetur.",

    "dialog_exit_unsaved_title": "Confirma Exitus",
    "dialog_exit_unsaved_text": "Apertum lima '{file_name}' servari non potest. Exitus procedere?",

    "message_app_config_file_access": "Permissio negata cum accedere ad configurationem applicationis limam in {file_path}. "
                                      "Constitue rectas permissiones ut operationem debitam asseguras.",

    "field_dir_path_dialog_caption": "Directory Select",
    "field_file_path_dialog_caption": "File Select",

    "load_file_encryption_password_mismatch": "Signi codicis discrepantia!",
    "load_file_encryption_password_incorrect": "Signum codicis incorrectum!",
    "load_file_none_content_error": "Fasciculum onerare non potest.",

    "action_new_file_error_occurred": "Filem creare non potest; error occurrit.\nPermissiones systematis file inspice.",
    "save_active_file_error_occurred": "Filem servare non potest; error occurrit.",

    "expandable_block_default_title": "Amplius info...",
    "expandable_block_open_close_tags_mismatch_warning": "<details> claudit/aperit block adsigna discongruitas",

    "dialog_color_picker_color_copied_to_the_clipboard": "Textus formatus in tabulam appensionis est copiatus",

    "popup_about_title": "De Applicatione",
    "popup_about_app_name_description": "Editor Markdown Pythonis",

    "popup_about_version": "Versio",
    "popup_about_license": "Licentia",
    "popup_about_website": "Situs",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "Dies",

    "update_helper_new_version_is_available": "Nova versio {latest_version} applicationis praesto est",
    "update_helper_latest_version_installed": "Ultima versio applicationis installata est",

    "network_connection_error_empty": "Informationes responsi obtineri non possunt",
    "network_connection_error_connection_or_dns":
        "Hospes non inventus est. Problema cum nexu interretiali aut DNS esse potest.",
    "network_connection_error_connection_refused":
        "Coniunctio recusata est. Servus deorsum ire potest, vel sunt problemae retis.",
    "network_connection_error_connection_timed_out": "Tempus coniunctionis excedit. Problemae retis esse possunt.",
    "network_connection_error_connection_404_error":
        "Error coniunctionis 404. Pagina vel res quaesita non inventa est.",
    "network_connection_error_generic_with_status_code": "Postulatio defecit cum statu codice: {status_code}",
}



# English lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "Select a Color",

    "color_picker_color_black": "Black",
    "color_picker_color_silver": "Silver",
    "color_picker_color_gray": "Gray",
    "color_picker_color_white": "White",
    "color_picker_color_maroon": "Maroon",
    "color_picker_color_red": "Red",
    "color_picker_color_purple": "Purple",
    "color_picker_color_fuchsia": "Fuchsia",
    "color_picker_color_green": "Green",
    "color_picker_color_lime": "Lime",
    "color_picker_color_olive": "Olive",
    "color_picker_color_yellow": "Yellow",
    "color_picker_color_navy": "Navy",
    "color_picker_color_blue": "Blue",
    "color_picker_color_teal": "Teal",
    "color_picker_color_aqua": "Aqua",
    "color_picker_color_aliceblue": "Alice Blue",
    "color_picker_color_antiquewhite": "Antique White",
    "color_picker_color_aquamarine": "Aquamarine",
    "color_picker_color_azure": "Azure",
    "color_picker_color_beige": "Beige",
    "color_picker_color_bisque": "Bisque",
    "color_picker_color_blanchedalmond": "Blanched Almond",
    "color_picker_color_blueviolet": "Blue Violet",
    "color_picker_color_brown": "Brown",
    "color_picker_color_burlywood": "Burly Wood",
    "color_picker_color_cadetblue": "Cadet Blue",
    "color_picker_color_chartreuse": "Chartreuse",
    "color_picker_color_chocolate": "Chocolate",
    "color_picker_color_coral": "Coral",
    "color_picker_color_cornflowerblue": "Cornflower Blue",
    "color_picker_color_cornsilk": "Cornsilk",
    "color_picker_color_crimson": "Crimson",
    "color_picker_color_cyan": "Cyan",
    "color_picker_color_darkblue": "Dark Blue",
    "color_picker_color_darkcyan": "Dark Cyan",
    "color_picker_color_darkgoldenrod": "Dark Golden Rod",
    "color_picker_color_darkgray": "Dark Gray",
    "color_picker_color_darkgreen": "Dark Green",
    "color_picker_color_darkkhaki": "Dark Khaki",
    "color_picker_color_darkmagenta": "Dark Magenta",
    "color_picker_color_darkolivegreen": "Dark Olive Green",
    "color_picker_color_darkorange": "Dark Orange",
    "color_picker_color_darkorchid": "Dark Orchid",
    "color_picker_color_darkred": "Dark Red",
    "color_picker_color_darksalmon": "Dark Salmon",
    "color_picker_color_darkseagreen": "Dark Sea Green",
    "color_picker_color_darkslateblue": "Dark Slate Blue",
    "color_picker_color_darkslategray": "Dark Slate Gray",
    "color_picker_color_darkturquoise": "Dark Turquoise",
    "color_picker_color_darkviolet": "Dark Violet",
    "color_picker_color_deeppink": "Deep Pink",
    "color_picker_color_deepskyblue": "Deep Sky Blue",
    "color_picker_color_dimgray": "Dim Gray",
    "color_picker_color_dodgerblue": "Dodger Blue",
    "color_picker_color_firebrick": "Fire Brick",
    "color_picker_color_floralwhite": "Floral White",
    "color_picker_color_forestgreen": "Forest Green",
    "color_picker_color_gainsboro": "Gainsboro",
    "color_picker_color_ghostwhite": "Ghost White",
    "color_picker_color_gold": "Gold",
    "color_picker_color_goldenrod": "Golden Rod",
    "color_picker_color_greenyellow": "Green Yellow",
    "color_picker_color_honeydew": "Honey Dew",
    "color_picker_color_hotpink": "Hot Pink",
    "color_picker_color_indianred": "Indian Red",
    "color_picker_color_indigo": "Indigo",
    "color_picker_color_ivory": "Ivory",
    "color_picker_color_khaki": "Khaki",
    "color_picker_color_lavender": "Lavender",
    "color_picker_color_lavenderblush": "Lavender Blush",
    "color_picker_color_lawngreen": "Lawn Green",
    "color_picker_color_lemonchiffon": "Lemon Chiffon",
    "color_picker_color_lightblue": "Light Blue",
    "color_picker_color_lightcoral": "Light Coral",
    "color_picker_color_lightcyan": "Light Cyan",
    "color_picker_color_lightgoldenrodyellow": "Light Golden Rod Yellow",
    "color_picker_color_lightgray": "Light Gray",
    "color_picker_color_lightgreen": "Light Green",
    "color_picker_color_lightpink": "Light Pink",
    "color_picker_color_lightsalmon": "Light Salmon",
    "color_picker_color_lightseagreen": "Light Sea Green",
    "color_picker_color_lightskyblue": "Light Sky Blue",
    "color_picker_color_lightslategray": "Light Slate Gray",
    "color_picker_color_lightsteelblue": "Light Steel Blue",
    "color_picker_color_lightyellow": "Light Yellow",
    "color_picker_color_limegreen": "Lime Green",
    "color_picker_color_linen": "Linen",
    "color_picker_color_magenta": "Magenta",
    "color_picker_color_mediumaquamarine": "Medium Aqua Marine",
    "color_picker_color_mediumblue": "Medium Blue",
    "color_picker_color_mediumorchid": "Medium Orchid",
    "color_picker_color_mediumpurple": "Medium Purple",
    "color_picker_color_mediumseagreen": "Medium Sea Green",
    "color_picker_color_mediumslateblue": "Medium Slate Blue",
    "color_picker_color_mediumspringgreen": "Medium Spring Green",
    "color_picker_color_mediumturquoise": "Medium Turquoise",
    "color_picker_color_mediumvioletred": "Medium Violet Red",
    "color_picker_color_midnightblue": "Midnight Blue",
    "color_picker_color_mintcream": "Mint Cream",
    "color_picker_color_mistyrose": "Misty Rose",
    "color_picker_color_moccasin": "Moccasin",
    "color_picker_color_navajowhite": "Navajo White",
    "color_picker_color_oldlace": "Old Lace",
    "color_picker_color_olivedrab": "Olive Drab",
    "color_picker_color_orange": "Orange",
    "color_picker_color_orangered": "Orange Red",
    "color_picker_color_orchid": "Orchid",
    "color_picker_color_palegoldenrod": "Pale Golden Rod",
    "color_picker_color_palegreen": "Pale Green",
    "color_picker_color_paleturquoise": "Pale Turquoise",
    "color_picker_color_palevioletred": "Pale Violet Red",
    "color_picker_color_papayawhip": "Papaya Whip",
    "color_picker_color_peachpuff": "Peach Puff",
    "color_picker_color_peru": "Peru",
    "color_picker_color_pink": "Pink",
    "color_picker_color_plum": "Plum",
    "color_picker_color_powderblue": "Powder Blue",
    "color_picker_color_rebeccapurple": "Rebecca Purple",
    "color_picker_color_rosybrown": "Rosy Brown",
    "color_picker_color_royalblue": "Royal Blue",
    "color_picker_color_saddlebrown": "Saddle Brown",
    "color_picker_color_salmon": "Salmon",
    "color_picker_color_sandybrown": "Sandy Brown",
    "color_picker_color_seagreen": "Sea Green",
    "color_picker_color_seashell": "Sea Shell",
    "color_picker_color_sienna": "Sienna",
    "color_picker_color_skyblue": "Sky Blue",
    "color_picker_color_slateblue": "Slate Blue",
    "color_picker_color_slategray": "Slate Gray",
    "color_picker_color_snow": "Snow",
    "color_picker_color_springgreen": "Spring Green",
    "color_picker_color_steelblue": "Steel Blue",
    "color_picker_color_tan": "Tan",
    "color_picker_color_thistle": "Thistle",
    "color_picker_color_tomato": "Tomato",
    "color_picker_color_turquoise": "Turquoise",
    "color_picker_color_violet": "Violet",
    "color_picker_color_wheat": "Wheat",
    "color_picker_color_whitesmoke": "White Smoke",
    "color_picker_color_yellowgreen": "Yellow Green",
}



# Russian lexemes ai_assistant.py
lexemes = {
    # ИИ помощник
    "dialog_title": "Запрос к ИИ",
    "dialog_prompt_input_placeholder_text": "Запрос",
    "dialog_prompt_input_accessible_description": "Введите запрос, чтобы дать контекст ИИ-помощнику",
    "dialog_response_output_placeholder_text": "Данные ответа",
    "dialog_response_output_accessible_description": "Данные ответа появятся в этом поле",
    "dialog_response_output_notice_empty_text": "Введите текст, чтобы получить ответ примерно...",

    "dialog_usage_module_label": "МОДУЛЬ",
    "dialog_usage_model_label": "МОДЕЛЬ",
    "dialog_usage_tokens_label": "ТОКЕНЫ",
    "dialog_usage_tokens_prompt": "запрос: {tokens}",
    "dialog_usage_tokens_answer": "ответ: {tokens}",
    "dialog_usage_tokens_total": "всего: {tokens}",

    "dialog_button_send_request": "Отправить запрос",
    "dialog_button_stop_request": "Стоп",
    "dialog_button_save_history": "Сохранить историю диалога",

    "dialog_message_copy_tooltip": "Копировать текст",
    "dialog_message_copied_tooltip": "Скопировано",

    "dialog_error_loading_model": "Ошибка загрузки пользовательской модели",
}



# English lexemes ai_assistant.py
lexemes = {
    # AI assistant dialog
    "dialog_title": "Ask the AI",
    "dialog_prompt_input_placeholder_text": "Prompt",
    "dialog_prompt_input_accessible_description": "Input request to give a context to the AI-assistant",
    "dialog_response_output_placeholder_text": "Response data",
    "dialog_response_output_accessible_description": "Response data will appear at this field",
    "dialog_response_output_notice_empty_text": "Provide a text input to get a response of about...",

    "dialog_usage_module_label": "MODULE",
    "dialog_usage_model_label": "MODEL",
    "dialog_usage_tokens_label": "TOKENS",
    "dialog_usage_tokens_prompt": "prompt: {tokens}",
    "dialog_usage_tokens_answer": "answer: {tokens}",
    "dialog_usage_tokens_total": "total: {tokens}",

    "dialog_button_send_request": "Ask",
    "dialog_button_stop_request": "Stop",
    "dialog_button_save_history": "Save dialog history",

    "dialog_message_copy_tooltip": "Copy text",
    "dialog_message_copied_tooltip": "Copied",

    "dialog_prompt_history_file_header": "# Ai Assistant [{datetime}]",

    "dialog_error_loading_model": "Error loading custom model",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Signal, QUrl
from PySide6.QtNetwork import QNetworkAccessManager, QNetworkReply, QNetworkRequest

from notolog.image_downloader import ImageDownloader
from notolog.settings import Settings
from notolog.app_config import AppConfig

from . import test_core_app  # noqa: F401

from unittest.mock import AsyncMock, MagicMock, patch
from pytest_mock import MockerFixture

import pytest
import asyncio
import logging


class TestImageDownloader:

    @staticmethod
    async def cleanup_tasks():
        # Cancel and clean up all pending asyncio tasks
        for task in asyncio.all_tasks():
            if task is not asyncio.current_task():
                task.cancel()
                try:
                    await task
                except asyncio.CancelledError:
                    pass

    @pytest.fixture()
    def test_settings_fixture(self, monkeypatch, request, test_core_app):  # noqa: F811 redefinition of unused 'test_app'
        # Retrieve parameter values from the test request.
        # The only 'viewer_save_resources' is in use at the moment
        monkeypatch.setattr(Settings, 'viewer_save_resources', request.param)

        # The test_core_app fixture sets up the app environment explicitly.
        yield Settings()

    def test_initialize_called(self):
        with patch.object(ImageDownloader, 'initialize', MagicMock(name='initialize')) as mock_initialize:
            ImageDownloader()
            # Ensure initialize was called once in the constructor
            mock_initialize.assert_called_once()

        with (patch.object(ImageDownloader, 'update_resource_folder', MagicMock(name='update_resource_folder'))
              as mock_update_resource_folder):
            ImageDownloader()
            # Ensure initialize was called once in the constructor
            mock_update_resource_folder.assert_called_once()

        with (patch.object(ImageDownloader, 'get_resource_folder', MagicMock(name='get_resource_folder'))
              as mock_get_resource_folder):
            ImageDownloader()
            # Ensure initialize was called once in the constructor
            mock_get_resource_folder.assert_called_once()

    @pytest.fixture
    def test_downloader_obj(self, mocker: MockerFixture):
        """
        Testing object fixture.
        May contain minimal logic for testing purposes.
        """

        # Mock AppConfig's get_logger_level method to suppress logging during tests.
        mocker.patch.object(AppConfig, 'get_logger_level', return_value=logging.NOTSET)

        yield ImageDownloader()

        # await obj_mock.cancel_tasks()

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @staticmethod
    def add_mock_signal(callback):
        signal = MagicMock(spec=Signal)
        # signal.emit = MagicMock(name='emit')
        # Explicitly add 'connect' to the signal
        signal.connect = lambda v: callback()
        signal.disconnect = lambda: None

        return signal

    @pytest.mark.asyncio
    @pytest.mark.parametrize(
        "test_settings_fixture, test_exp_params_fixture, test_downloader_obj",
        [
            (False, (QNetworkReply.NetworkError.NoError, None, 'some_url1',
                     {QNetworkRequest.KnownHeaders.ContentTypeHeader: 'image'}, b'image_byte_array_data'), None),
            (True, (QNetworkReply.NetworkError.NoError, None, 'some_url1',
                    {QNetworkRequest.KnownHeaders.ContentTypeHeader: 'image'}, b'image_byte_array_data'), None),
            (False, (QNetworkReply.NetworkError.HostNotFoundError, 404, 'some_url123',
                     {QNetworkRequest.KnownHeaders.ContentTypeHeader: 'anything'}, b'image_byte_array_data'), None),
        ],
        # This tells pytest that the parameter in a "..." should not be passed directly to the test function.
        # indirect=["test_downloader_obj"]
        indirect=True  # all
    )
    async def test_async_queue(self, mocker: MockerFixture, monkeypatch,
                               test_settings_fixture: Settings,
                               test_exp_params_fixture: dict,
                               test_downloader_obj: ImageDownloader):
        # Test params
        reply_error, error_code, resource_url, header_params, resource_data = test_exp_params_fixture

        # Get callback method to check
        mock_handle_network_reply = mocker.patch.object(
            test_downloader_obj, 'handle_network_reply', wraps=test_downloader_obj.handle_network_reply)
        mock_handle_network_reply_error = mocker.patch.object(
            test_downloader_obj, 'handle_network_reply_error',
            wraps=test_downloader_obj.handle_network_reply_error)
        # Download in async queue
        mock_download_resource_in_queue = mocker.patch.object(
            test_downloader_obj, 'download_resource_in_queue', new_callable=AsyncMock,
            wraps=test_downloader_obj.download_resource_in_queue)
        mock_resource_download_async = mocker.patch.object(
            test_downloader_obj, 'resource_download_async', wraps=test_downloader_obj.resource_download_async)
        # Get download callback method to check
        mock_callback = mocker.patch.object(
            test_downloader_obj, 'resource_task_callback', wraps=test_downloader_obj.resource_task_callback)
        # Downloading resource
        mock_download_image = mocker.patch.object(
            test_downloader_obj, 'download_image', wraps=test_downloader_obj.download_image)
        # Cache and save downloaded resource
        mock_save_image = mocker.patch.object(test_downloader_obj, 'save_image', return_value=None)
        mock_cache_pixmap = mocker.patch.object(test_downloader_obj, 'cache_pixmap', return_value=None)

        # AsyncMock mock class is designed to simulate asynchronous calls and can be awaited.
        mocker.patch.object(asyncio, 'wait', AsyncMock(return_value=(None, None)))

        # Check that the local task pool is empty.
        # An error here indicates that the queue was used in previous tests.
        assert len(test_downloader_obj.resource_tasks) == 0
        # Check default resource folder
        assert test_downloader_obj.folder.path().endswith(ImageDownloader.RESOURCE_DIR)

        # Mock QNetworkReply using create_autospec
        # mock_reply = create_autospec(QNetworkReply, instance=True)
        mock_reply = MagicMock(spec=QNetworkReply)
        mock_reply.error.return_value = reply_error
        mock_reply.readAll.return_value = resource_data
        mock_reply.url.return_value = QUrl(resource_url)
        assert mock_reply.url().toString() == resource_url

        # with patch('PySide6.QtGui.QPixmap') as MockPixmap:
        #    mock_pixmap_instance = MockPixmap.return_value
        #    mock_pixmap_instance.loadFromData = MagicMock()  # MagicMock(spec=QPixmap)

        mocker.patch.object(mock_reply, 'header', side_effect=lambda k: header_params.get(k, None))

        # Add 'finished' signal to mock_reply with an 'emit' method
        mock_reply.finished = self.add_mock_signal(
            lambda: test_downloader_obj.handle_network_reply(mock_reply))
        mock_reply.errorOccurred = self.add_mock_signal(
            lambda: test_downloader_obj.handle_network_reply_error(mock_reply, error_code))

        # Mock QNetworkAccessManager and its get method
        mock_manager = MagicMock(QNetworkAccessManager)
        mock_manager.get.return_value = mock_reply

        # Monkeypatch the network manager in the application
        monkeypatch.setattr(test_downloader_obj, 'network_manager', mock_manager)
        # monkeypatch.setattr("PySide6.QtNetwork.QNetworkAccessManager", lambda: mock_manager)

        mocker.patch.object(test_downloader_obj, 'is_external_url', return_value=True)

        await test_downloader_obj.download_resource_in_queue(resource_url)
        assert test_downloader_obj.downloaded_cnt == 0

        await asyncio.sleep(0.025)  # Faster sleeps may not work on macOS or Windows test environments

        mock_callback.assert_called_once()
        mock_download_resource_in_queue.assert_awaited_once_with(resource_url)

        await test_downloader_obj.download_resource_in_queue(resource_url)
        await asyncio.sleep(0.05)  # Or 0.251, slightly more than described in a queue method, to get all completed

        assert mock_callback.call_count == 2

        def _check_downloaded(url):
            assert 'resource_name' in url
            assert url['resource_name'] == resource_url
            assert test_downloader_obj.downloaded_cnt == 1
            assert len(test_downloader_obj.resource_tasks) == 0
        # Assert 'downloaded' signal
        test_downloader_obj.downloaded.connect(_check_downloaded)

        # Assert status code was checked
        mock_reply.attribute.assert_called_with(QNetworkRequest.Attribute.HttpStatusCodeAttribute)

        if error_code is None:
            mock_handle_network_reply.assert_called()
            mock_resource_download_async.assert_called_with(resource_url)
            mock_resource_download_async.assert_awaited_with(resource_url)
            mock_download_image.assert_called_with(resource_url)
            mock_cache_pixmap.assert_called_with(resource_url, resource_data)
        else:
            mock_handle_network_reply_error.assert_called()

        assert test_downloader_obj.settings.viewer_save_resources == test_settings_fixture.viewer_save_resources
        if test_downloader_obj.settings.viewer_save_resources:
            mock_save_image.assert_called()

        # The cleanup async tasks to speed up checks and exit
        await self.cleanup_tasks()

    @pytest.mark.parametrize(
        "test_exp_params_fixture, test_downloader_obj",
        [
            (('some_url1', 'some_exp|ected_hash', 'some_exp_some_url1'), None),
            (('some_url2', 'some_exp|ected_hash', 'some_exp_some_url2'), None),
            (('notolog.app/example.png', 'some_exp|ected_hash', 'some_exp_example.png'), None),
        ],
        indirect=True
    )
    def test_url_to_filename(self, monkeypatch, test_exp_params_fixture, test_downloader_obj: ImageDownloader):
        # Test params
        resource_url, hash256, expected_filename = test_exp_params_fixture

        # Mocking hashlib.sha256().hexdigest() using patch
        with patch('hashlib.sha256') as mock_sha256:
            # Mocking the return value of hexdigest() method
            mock_sha256.return_value.hexdigest.return_value = hash256

            assert test_downloader_obj.url_to_filename(resource_url) == expected_filename

    @pytest.mark.parametrize(
        "test_exp_params_fixture, test_downloader_obj",
        [
            (('image/jpeg', '.jpg'), None),
            (('image/png', '.png'), None),
            (('image/gif', '.gif'), None),
            (('image/bmp', '.bmp'), None),
            (('image/svg+xml', '.svg'), None),
            (('image/tiff', '.tiff'), None),
            (('image/webp', '.webp'), None),
            (('image/anything-else', '.dat'), None),
        ],
        indirect=True
    )
    def test_mime_to_extension(self, test_exp_params_fixture, test_downloader_obj: ImageDownloader):
        # Test params
        mime_type, file_extension = test_exp_params_fixture

        assert test_downloader_obj.mime_to_extension(mime_type) == file_extension



# Spanish lexemes main_menu.py
lexemes = {
    "group_file_label": "Archivo",
    "group_edit_label": "Editar",
    "group_tools_label": "Herramientas",
    "group_help_label": "Ayuda",

    "actions_file_label_new_document": "Nuevo documento",
    "actions_file_accessible_name_new_document": "Nuevo documento",
    "actions_file_label_open": "Abrir",
    "actions_file_accessible_name_open": "Abrir documento",
    "actions_file_label_save": "Guardar",
    "actions_file_accessible_name_save": "Guardar documento",
    "actions_file_label_save_as": "Guardar como",
    "actions_file_accessible_name_save_as": "Guardar como copia del documento",
    "actions_file_label_settings": "Configuración",
    "actions_file_accessible_name_settings": "Configuración de la aplicación",
    "actions_file_label_reset_settings": "Restablecer configuraciones",
    "actions_file_accessible_name_reset_settings": "Restablecer configuraciones",
    "actions_file_label_exit": "Salir",
    "actions_file_accessible_name_exit": "Salir de la aplicación",

    "actions_edit_edit_mode": "Editar",
    "actions_edit_accessible_name_edit_mode": "Editar archivo",
    "actions_edit_source_mode": "Fuente",
    "actions_edit_accessible_name_source_mode": "Código fuente",
    "actions_edit_bold": "Negrita",
    "actions_edit_accessible_name_bold": "Formato de texto en negrita",
    "actions_edit_italic": "Cursiva",
    "actions_edit_accessible_name_italic": "Formato de texto en cursiva",
    "actions_edit_underline": "Subrayado",
    "actions_edit_accessible_name_underline": "Formato de texto subrayado",
    "actions_edit_strikethrough": "Tachado",
    "actions_edit_accessible_name_strikethrough": "Formato de texto tachado",
    "actions_edit_blockquote": "Cita",
    "actions_edit_accessible_name_blockquote": "Formato de texto de cita",

    "actions_tools_ai_assistant": "Asistente de IA",
    "actions_tools_accessible_name_ai_assistant": "Lanzar el asistente de IA",
    "actions_tools_color_picker": "Selector de color",
    "actions_tools_accessible_name_color_picker": "Elegir un color de texto",

    "actions_help_label_md_syntax": "Sintaxis Markdown",
    "actions_help_accessible_name_md_syntax": "Sintaxis Markdown",
    "actions_help_label_check_for_updates": "Buscar actualizaciones",
    "actions_help_accessible_name_check_for_updates": "Buscar actualizaciones",
    "actions_help_label_bug_report": "Reportar error",
    "actions_help_accessible_name_bug_report": "Enviar reporte de error",
    "actions_help_label_about": "Acerca de",
    "actions_help_accessible_name_about": "Acerca de",
}



# Swedish lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "Modul llama.cpp",

    "module_llama_cpp_config_label": "Modul llama.cpp",
    "module_llama_cpp_config_path_label": "Modellplats",
    "module_llama_cpp_config_path_input_placeholder_text": "Välj eller ange modellens sökväg",
    "module_llama_cpp_config_path_input_accessible_description":
        "Ett inmatningsfält med en väljare för att specificera sökvägen för den lokala modellen.\n"
        "Stöder modeller i GGUF-format, ett binärt filformat optimerat för att lagra\n"
        "modeller som används med GGML och GGML-baserade verkställare.",
    "module_llama_cpp_config_path_input_filter_text": "GGUF-filer",

    "module_llama_cpp_config_context_window_label": "Kontextfönstrets storlek",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "Anger antalet tokens som modellen överväger för att generera svar.\n"
        "Kontrollerar hur mycket tidigare kontext som används.",

    "module_llama_cpp_chat_formats_label": "Chatformat",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "Välj ett chattformat",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "Dropdown-menyn för att välja formatet som används för modellsamtal.",

    "module_llama_cpp_config_system_prompt_label": "Systemprompt",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "Ange text för systemprompten",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "Textfält för att ange systemprompter som styr modellens svar.",

    "module_llama_cpp_config_response_temperature_label": "Svarstemperatur: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "Justerar modellens svars slumpmässighet. Högre värden producerar mer varierade utdata,\n"
        "medan lägre värden resulterar i mer förutsägbara svar.",

    "module_llama_cpp_config_response_max_tokens_label": "Maximalt antal tokens per svar",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "Begränsar antalet tokens i modellens svar till den faktiska gränsen för kontextfönstret.\n"
        "Ett värde på noll antar kapaciteten för kontextfönstret.",

    "module_llama_cpp_config_prompt_history_size_label": "Storleken på promptens historia",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "Kontrollerar antalet poster i promptens historik som systemet behåller för referens.\n"
        "Ett värde på noll tillåter obegränsat antal poster."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.app_config import AppConfig
from notolog.helpers.settings_helper import SettingsHelper

from logging import Logger, NOTSET

import os
import pytest
import base64


class TestSettingsHelper:

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_settings_helper(self, mocker, request):
        # Retrieve parameter values from the test request.
        env_values, app_config_key, generated_key = request.param

        # Mock AppConfig's get_logger_level method to suppress logging during tests.
        mocker.patch.object(AppConfig, 'get_logger_level', return_value=NOTSET)

        # Mock methods result
        mocker.patch.object(os, 'getenv', side_effect=lambda k: env_values.get(k, None))
        mocker.patch.object(AppConfig, 'get_security_app_secret', return_value=app_config_key)
        mocker.patch.object(SettingsHelper, 'generate_app_key', return_value=generated_key)
        # Do not call it from test to avoid data loss
        mocker.patch.object(SettingsHelper, 'setup_app_key', return_value=None)

        yield SettingsHelper()

    @pytest.fixture(scope="function")
    def test_exp_param_fixture(self, request):
        # Get the parameter value from the request
        param_value = request.param

        yield param_value

    @pytest.mark.parametrize(
        "test_obj_settings_helper, test_exp_param_fixture",
        [
            (({SettingsHelper.env_secret_name: None}, None, b''), b''),
            (({SettingsHelper.env_secret_name: 'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='}, None, b''),
             b'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='),  # 32 len bytes key
            (({SettingsHelper.env_secret_name: None}, 'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ=', b''),
             b'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='),
            (({SettingsHelper.env_secret_name: None}, None, b'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='),
             b'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='),
            (({SettingsHelper.env_secret_name: 'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='}, 'ANYKEY1', b''),
             b'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='),
            (({SettingsHelper.env_secret_name: 'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='}, 'ANYKEY2', b'ANYKEY3'),
             b'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='),
        ],
        indirect=True
    )
    def test_settings_helper_init(self, test_obj_settings_helper: SettingsHelper, test_exp_param_fixture):
        # Check initial params are exist
        assert isinstance(test_obj_settings_helper.logger, Logger)

        assert hasattr(test_obj_settings_helper, 'key')
        assert test_obj_settings_helper.key == test_exp_param_fixture

    @pytest.mark.parametrize(
        "test_obj_settings_helper, test_exp_param_fixture",
        [
            (({SettingsHelper.env_secret_name: 'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='}, 'ANYKEY0', b'ANYKEY1'),
             'test_data'),
        ],
        indirect=True
    )
    def test_encrypt_data(self, test_obj_settings_helper: SettingsHelper, test_exp_param_fixture):
        encrypted_data = test_obj_settings_helper.encrypt_data(test_exp_param_fixture)
        assert test_obj_settings_helper.decrypt_data(encrypted_data) == test_exp_param_fixture  # type: ignore

    @pytest.mark.parametrize(
        "test_obj_settings_helper, test_exp_param_fixture",
        [
            (({SettingsHelper.env_secret_name: 'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='}, 'ANYKEY2', b'ANYKEY3'),
             b'gAAAAABmS0svwU1fjdaizp0LU89WSvLBodh1RLR5E2EGSNp9wt3PZfFZsIrc_N_NuKyws7uC-BGtd0iqIsCTbqSuf7PaZf2oGQ=='),
        ],
        indirect=True
    )
    def test_decrypt_data(self, test_obj_settings_helper: SettingsHelper,
                          test_exp_param_fixture):
        assert test_obj_settings_helper.decrypt_data(test_exp_param_fixture) == 'test_data'  # type: ignore

    @pytest.mark.parametrize(
        "test_obj_settings_helper, test_exp_param_fixture",
        [
            (({SettingsHelper.env_secret_name: 'c29tZWtleTFzb21la2V5MnNvbWVrZXkzc29tZWtleTQ='}, 'ANYKEY0', b'ANYKEY1'),
             b'somekey1somekey2somekey3somekey4'),
        ],
        indirect=True
    )
    def test_get_app_key(self, test_obj_settings_helper: SettingsHelper, test_exp_param_fixture):
        # Try to decode the key string
        decoded_bytes = base64.b64decode(test_obj_settings_helper.key, validate=True)
        assert decoded_bytes == test_exp_param_fixture
        # If decoding is successful, check if encoding back gives the same string
        # This ensures padding and character checks are adhered to
        assert base64.b64encode(decoded_bytes) == test_obj_settings_helper.key

    @pytest.mark.parametrize(
        "test_obj_settings_helper, test_exp_param_fixture",
        [
            (({}, None, b''), ('', '', '')),
            (({}, None, b''), ('object_name1', 'object_name1', 'object_name1')),
            (({}, None, b''), ('lexeme1:key1', 'lexeme1', 'key1')),
            (({}, None, b''), (':key1', '', 'key1')),
        ],
        indirect=True
    )
    def test_parse_object_name(self, test_obj_settings_helper: SettingsHelper, test_exp_param_fixture):
        # Get testing parameters
        object_name, exp_lexeme, exp_setting_key = test_exp_param_fixture
        # Parse the object name in case it contains a combination of lexeme and setting keys
        lexeme, setting_key = test_obj_settings_helper.parse_object_name(object_name)
        assert lexeme == exp_lexeme
        assert setting_key == exp_setting_key



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Enter password UI dialog.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QDialog, QLabel, QLineEdit, QPushButton, QHBoxLayout, QVBoxLayout, QStyle
from PySide6.QtGui import QFontMetrics, QFont

from . import Settings
from . import Lexemes
from . import ThemeHelper

import logging


class EncPasswordDialog(QDialog):

    def __init__(self, hint: str = None, parent=None):
        super().__init__(parent)

        self.parent = parent

        if self.parent and hasattr(self.parent, 'font'):
            # Apply font from the main window to the dialog
            self.setFont(self.parent.font())

        self.settings = Settings(parent=self)

        self.logger = logging.getLogger('enc_password_dialog')

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='common')

        self.theme_helper = ThemeHelper()

        title = self.lexemes.get('dialog_encrypt_password_title')
        self.setWindowTitle(title)

        password_layout = QVBoxLayout()
        self.password_edit = QLineEdit()
        password_layout.addWidget(QLabel(self.lexemes.get('dialog_encrypt_password_label')))
        self.password_edit.setPlaceholderText(self.lexemes.get('dialog_encrypt_password_input_placeholder_text'))
        self.password_edit.setEchoMode(QLineEdit.EchoMode.Password)
        self.password_edit.setFocus()
        password_layout.addWidget(self.password_edit)

        # Hint
        hint_layout = QVBoxLayout()
        hint_layout.addWidget(QLabel(self.lexemes.get('dialog_encrypt_password_hint_label')))
        hint_description = QLabel(hint)
        hint_description.setFont(QFont(hint_description.font().family(), italic=True))
        hint_description.setStyleSheet("color: %s" % self.theme_helper.get_color('dialog_encrypt_password_hint',
                                                                                 css_format=True))
        hint_layout.addWidget(hint_description)

        buttons_layout = QHBoxLayout()

        self.ok_button = QPushButton(self.lexemes.get('dialog_encrypt_password_button_ok'))
        self.ok_button.setIcon(self.style().standardIcon(QStyle.StandardPixmap.SP_DialogOkButton))
        self.ok_button.clicked.connect(self.accept)
        self.ok_button.setEnabled(False)

        # https://doc.qt.io/qt-6/qt.html#FocusPolicy-enum
        self.cancel_button = QPushButton(self.lexemes.get('dialog_encrypt_password_button_cancel'),
                                         focusPolicy=Qt.FocusPolicy.NoFocus)
        self.cancel_button.setIcon(self.style().standardIcon(QStyle.StandardPixmap.SP_DialogCancelButton))
        self.cancel_button.clicked.connect(self.reject)

        buttons_layout.addWidget(self.cancel_button)
        buttons_layout.addWidget(self.ok_button)

        layout = QVBoxLayout()
        layout.addLayout(password_layout)
        if hint:
            layout.addLayout(hint_layout)
        layout.addLayout(buttons_layout)

        self.setLayout(layout)

        self.adjust_minimum_width(title)

        self.password_edit.textChanged.connect(self.check_input_length)

    def check_input_length(self):
        password_length = len(self.password_edit.text())

        # Enable OK button only if field has a non-zero length
        self.ok_button.setEnabled(password_length > 0)

    def adjust_minimum_width(self, title):
        # Calculate width required for the title text
        font_metrics = QFontMetrics(self.font())
        title_width = font_metrics.horizontalAdvance(title)

        # Set minimum width for the dialog
        self.setMinimumWidth(title_width + 150)  # Add some padding



# Russian lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "LLM на устройстве",

    "module_ondevice_llm_model_exception": "Ошибка: модель не найдена: {error_msg}",
    "module_ondevice_llm_task_exception": "Не удается инициировать генератор модели: {error_msg}",
}



# Korean lexemes common.py
lexemes = {
    "module_llama_cpp_name": "모듈 llama.cpp",

    "module_llama_cpp_model_exception": "모델을 찾을 수 없음 오류: {error_msg}",
    "module_llama_cpp_task_exception": "모델 생성기를 초기화할 수 없음: {error_msg}",
}



# Portuguese lexemes ai_assistant.py
lexemes = {
    # Assistente de IA
    "dialog_title": "Solicitação à IA",
    "dialog_prompt_input_placeholder_text": "Solicitação",
    "dialog_prompt_input_accessible_description": "Insira a solicitação para fornecer contexto ao assistente de IA",
    "dialog_response_output_placeholder_text": "Dados da resposta",
    "dialog_response_output_accessible_description": "Os dados da resposta aparecerão neste campo",
    "dialog_response_output_notice_empty_text": "Forneça uma entrada de texto para obter uma resposta de cerca de...",

    "dialog_usage_module_label": "MÓDULO",
    "dialog_usage_model_label": "MODELO",
    "dialog_usage_tokens_label": "TOKENS",
    "dialog_usage_tokens_prompt": "solicitação: {tokens}",
    "dialog_usage_tokens_answer": "resposta: {tokens}",
    "dialog_usage_tokens_total": "total: {tokens}",

    "dialog_button_send_request": "Enviar solicitação",
    "dialog_button_stop_request": "Parar",
    "dialog_button_save_history": "Salvar histórico do diálogo",

    "dialog_message_copy_tooltip": "Copiar texto",
    "dialog_message_copied_tooltip": "Copiado",

    "dialog_error_loading_model": "Erro ao carregar modelo personalizado",
}



# Georgian lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "მოდული llama.cpp",

    "module_llama_cpp_config_label": "მოდული llama.cpp",
    "module_llama_cpp_config_path_label": "მოდელის მდებარეობა",
    "module_llama_cpp_config_path_input_placeholder_text": "აირჩიეთ ან შეიყვანეთ მოდელის გზა",
    "module_llama_cpp_config_path_input_accessible_description":
        "შეყვანის ველი სელექტორით ადგილზე მოდელის გზის მითითებისთვის. მხარდაჭერს GGUF ფორმატის მოდელებს,\n"
        "ბინარული ფაილის ფორმატი, ოპტიმიზირებული GGML-ით და GGML-ზე მუშაობადი ეგზეკუტორებისთვის.",
    "module_llama_cpp_config_path_input_filter_text": "GGUF ფაილები",

    "module_llama_cpp_config_context_window_label": "კონტექსტის ფანჯრის ზომა",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "აყენებს ტოკენების რაოდენობას, რომელსაც მოდელი გამოიყენებს პასუხების გენერირებისთვის. კონტროლირებს წინა კონტექსტის\n"
        "გამოყენების რაოდენობას.",

    "module_llama_cpp_chat_formats_label": "ჩეთის ფორმატები",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "აირჩიეთ ჩეთის ფორმატი",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "ჩამოშლილი მენიუ მოდელის საუბარების ფორმატის არჩევისთვის.",

    "module_llama_cpp_config_system_prompt_label": "სისტემის პრომპტი",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "შეიყვანეთ სისტემის პრომპტის ტექსტი",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "ტექსტის ველი, რომელიც განაცხადი პრომპტების შეყვანას რეგულირებს, რომლებიც იძლევა\n"
        "მოდელის პასუხების გამართვას.",

    "module_llama_cpp_config_response_temperature_label": "პასუხის ტემპერატურა: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "რეგულირებს მოდელის პასუხების შემთხვევითობას. მაღალი მნიშვნელობები ქმნიან უფრო ვარიაციულ გამოსავალებს,\n"
        "დაბალი მნიშვნელობები გამოიღებენ უფრო წინასწარ განსაზღვრავ პასუხებს.",

    "module_llama_cpp_config_response_max_tokens_label": "პასუხში მაქსიმუმ ტოკენების რაოდენობა",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "შეზღუდავს მოდელის პასუხებში ტოკენების რაოდენობას აქტუალური კონტექსტის ფანჯრის ლიმიტამდე.\n"
        "ნულოვანი მნიშვნელობა გამოიყენებს კონტექსტის ფანჯრის ჩათვლითი ტევადობას.",

    "module_llama_cpp_config_prompt_history_size_label": "პრომპტის ისტორიის ზომა",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "კონტროლირებს პრომპტების ისტორიაში ჩანაწერების რაოდენობას, რომელიც სისტემამ სარეფერენციოდ ინახავს.\n"
        "ნულოვანი მნიშვნელობა აძლევს ულიმიტო შეტანებას."
}



# Swedish lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "Välj en färg",

    "color_picker_color_black": "Svart",
    "color_picker_color_silver": "Silver",
    "color_picker_color_gray": "Grå",
    "color_picker_color_white": "Vit",
    "color_picker_color_maroon": "Rödbrun",
    "color_picker_color_red": "Röd",
    "color_picker_color_purple": "Lila",
    "color_picker_color_fuchsia": "Fuchsia",
    "color_picker_color_green": "Grön",
    "color_picker_color_lime": "Limegrön",
    "color_picker_color_olive": "Oliv",
    "color_picker_color_yellow": "Gul",
    "color_picker_color_navy": "Marinblå",
    "color_picker_color_blue": "Blå",
    "color_picker_color_teal": "Teal",
    "color_picker_color_aqua": "Akvamarin",
    "color_picker_color_aliceblue": "Aliceblå",
    "color_picker_color_antiquewhite": "Antikvit",
    "color_picker_color_aquamarine": "Aquamarine",
    "color_picker_color_azure": "Azurblå",
    "color_picker_color_beige": "Beige",
    "color_picker_color_bisque": "Bisque",
    "color_picker_color_blanchedalmond": "Skållad mandel",
    "color_picker_color_blueviolet": "Blåviolett",
    "color_picker_color_brown": "Brun",
    "color_picker_color_burlywood": "Burlywood",
    "color_picker_color_cadetblue": "Kadettblå",
    "color_picker_color_chartreuse": "Chartreuse",
    "color_picker_color_chocolate": "Choklad",
    "color_picker_color_coral": "Korall",
    "color_picker_color_cornflowerblue": "Kornblå",
    "color_picker_color_cornsilk": "Majssilke",
    "color_picker_color_crimson": "Karmosin",
    "color_picker_color_cyan": "Cyan",
    "color_picker_color_darkblue": "Mörkblå",
    "color_picker_color_darkcyan": "Mörk cyan",
    "color_picker_color_darkgoldenrod": "Mörk gyllenstång",
    "color_picker_color_darkgray": "Mörkgrå",
    "color_picker_color_darkgreen": "Mörkgrön",
    "color_picker_color_darkkhaki": "Mörk khaki",
    "color_picker_color_darkmagenta": "Mörk magenta",
    "color_picker_color_darkolivegreen": "Mörkolivgrön",
    "color_picker_color_darkorange": "Mörkorange",
    "color_picker_color_darkorchid": "Mörk orkidé",
    "color_picker_color_darkred": "Mörkröd",
    "color_picker_color_darksalmon": "Mörklax",
    "color_picker_color_darkseagreen": "Mörkt havsgrön",
    "color_picker_color_darkslateblue": "Mörk skifferblå",
    "color_picker_color_darkslategray": "Mörk skiffergrå",
    "color_picker_color_darkturquoise": "Mörkturkos",
    "color_picker_color_darkviolet": "Mörkviolett",
    "color_picker_color_deeppink": "Djuprosa",
    "color_picker_color_deepskyblue": "Djuphimmelsblå",
    "color_picker_color_dimgray": "Dimgrå",
    "color_picker_color_dodgerblue": "Dodgerblå",
    "color_picker_color_firebrick": "Mursten",
    "color_picker_color_floralwhite": "Blomstervit",
    "color_picker_color_forestgreen": "Skogsgrön",
    "color_picker_color_gainsboro": "Gainsboro",
    "color_picker_color_ghostwhite": "Spökvit",
    "color_picker_color_gold": "Guld",
    "color_picker_color_goldenrod": "Gyllenstång",
    "color_picker_color_greenyellow": "Gröngul",
    "color_picker_color_honeydew": "Honungsdagg",
    "color_picker_color_hotpink": "Hett rosa",
    "color_picker_color_indianred": "Indisk röd",
    "color_picker_color_indigo": "Indigo",
    "color_picker_color_ivory": "Elfenben",
    "color_picker_color_khaki": "Khaki",
    "color_picker_color_lavender": "Lavendel",
    "color_picker_color_lavenderblush": "Lavendelrosa",
    "color_picker_color_lawngreen": "Gräsmattagrön",
    "color_picker_color_lemonchiffon": "Citronchiffong",
    "color_picker_color_lightblue": "Ljusblå",
    "color_picker_color_lightcoral": "Ljuskorall",
    "color_picker_color_lightcyan": "Ljus cyan",
    "color_picker_color_lightgoldenrodyellow": "Ljus gyllenstångsgul",
    "color_picker_color_lightgray": "Ljusgrå",
    "color_picker_color_lightgreen": "Ljusgrön",
    "color_picker_color_lightpink": "Ljusrosa",
    "color_picker_color_lightsalmon": "Ljuslax",
    "color_picker_color_lightseagreen": "Ljust havsgrön",
    "color_picker_color_lightskyblue": "Ljus himmelsblå",
    "color_picker_color_lightslategray": "Ljus skiffergrå",
    "color_picker_color_lightsteelblue": "Ljust stålblå",
    "color_picker_color_lightyellow": "Ljusgul",
    "color_picker_color_limegreen": "Limegrön",
    "color_picker_color_linen": "Linnetyg",
    "color_picker_color_magenta": "Magenta",
    "color_picker_color_mediumaquamarine": "Mellan aquamarine",
    "color_picker_color_mediumblue": "Mellanblå",
    "color_picker_color_mediumorchid": "Mellan orkidé",
    "color_picker_color_mediumpurple": "Mellanlila",
    "color_picker_color_mediumseagreen": "Mellant havsgrön",
    "color_picker_color_mediumslateblue": "Mellan skifferblå",
    "color_picker_color_mediumspringgreen": "Mellant vårgrön",
    "color_picker_color_mediumturquoise": "Mellanturkos",
    "color_picker_color_mediumvioletred": "Mellan violetröd",
    "color_picker_color_midnightblue": "Midnattsblå",
    "color_picker_color_mintcream": "Mintkräm",
    "color_picker_color_mistyrose": "Dimrosa",
    "color_picker_color_moccasin": "Mockasin",
    "color_picker_color_navajowhite": "Navajovit",
    "color_picker_color_oldlace": "Gammal spets",
    "color_picker_color_olivedrab": "Olivtrist",
    "color_picker_color_orange": "Orange",
    "color_picker_color_orangered": "Orangeröd",
    "color_picker_color_orchid": "Orkidé",
    "color_picker_color_palegoldenrod": "Blek gyllenstång",
    "color_picker_color_palegreen": "Blekgrön",
    "color_picker_color_paleturquoise": "Blekturkos",
    "color_picker_color_palevioletred": "Blek violetröd",
    "color_picker_color_papayawhip": "Papayagrädde",
    "color_picker_color_peachpuff": "Persikopuff",
    "color_picker_color_peru": "Peru",
    "color_picker_color_pink": "Rosa",
    "color_picker_color_plum": "Plommon",
    "color_picker_color_powderblue": "Pulverblå",
    "color_picker_color_rebeccapurple": "Rebeccalila",
    "color_picker_color_rosybrown": "Rosenbrun",
    "color_picker_color_royalblue": "Kungsblå",
    "color_picker_color_saddlebrown": "Sadelbrun",
    "color_picker_color_salmon": "Lax",
    "color_picker_color_sandybrown": "Sandbrun",
    "color_picker_color_seagreen": "Havsgrön",
    "color_picker_color_seashell": "Snäckskal",
    "color_picker_color_sienna": "Sienna",
    "color_picker_color_skyblue": "Himmelsblå",
    "color_picker_color_slateblue": "Skifferblå",
    "color_picker_color_slategray": "Skiffergrå",
    "color_picker_color_snow": "Snö",
    "color_picker_color_springgreen": "Vårgrön",
    "color_picker_color_steelblue": "Stålblå",
    "color_picker_color_tan": "Tan",
    "color_picker_color_thistle": "Tistel",
    "color_picker_color_tomato": "Tomat",
    "color_picker_color_turquoise": "Turkos",
    "color_picker_color_violet": "Violett",
    "color_picker_color_wheat": "Vete",
    "color_picker_color_whitesmoke": "Vitrök",
    "color_picker_color_yellowgreen": "Gulgrön"
}



# Portuguese lexemes common.py
lexemes = {
    "module_llama_cpp_name": "Módulo llama.cpp",

    "module_llama_cpp_model_exception": "Erro de modelo não encontrado: {error_msg}",
    "module_llama_cpp_task_exception": "Não é possível iniciar o gerador de modelo: {error_msg}",
}



# Dutch lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "OpenAI API",

    "module_openai_api_label": "OpenAI API",
    "module_openai_api_url_label": "API URL",
    "module_openai_api_url_input_placeholder_text": "API URL",
    "module_openai_api_url_input_accessible_description":
        "De URL van de OpenAI API is het adres van het API-eindpunt, dat kan variëren afhankelijk van de service en versie.\n"
        "De AI-assistent gebruikt het eindpunt voor gesprekschatfunctionaliteit of tekstaanvullingen.\n"
        "Raadpleeg de officiële documentatie van de OpenAI API voor de actuele URL.",
    "module_openai_api_key_label": "API-sleutel",
    "module_openai_api_key_input_placeholder_text": "API-sleutel",
    "module_openai_api_key_input_accessible_description":
        "De API-sleutel van OpenAI is een geheime token die wordt gebruikt voor het authenticeren\n"
        "van verzoeken aan het API-eindpunt.",
    "module_openai_api_supported_models_label": "Ondersteunde modellen",
    "module_openai_api_model_names_combo_placeholder_text": "Kies een model",
    "module_openai_api_model_names_combo_accessible_description":
        "Selecteer uit ondersteunde modellen voor chatgesprekken.",

    "module_openai_api_base_system_prompt_label": "Systeemprompt",
    "module_openai_api_base_system_prompt_edit_placeholder_text": "Basis systeemprompt die elke aanvraag voorafgaat",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "Een basis systeemprompt die elke aanvraag voorafgaat.\n"
        "Dit is gewoonlijk een eenvoudige tekst met instructies of rolkenmerken.",

    "module_openai_api_base_response_temperature_label": "Temperatuur: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "Past de willekeurigheid van de modelresponsen aan. Hogere waarden produceren meer gevarieerde uitkomsten, "
        "terwijl lagere waarden de reacties voorspelbaarder maken.",

    "module_openai_api_base_response_max_tokens_label": "Maximaal aantal respons tokens",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "Het maximale aantal tokens dat ontvangen kan worden in een respons, zoals woorden en leestekens, "
        "regelt de lengte van de uitvoer.",

    "module_openai_api_config_prompt_history_size_label": "Grootte van de promptgeschiedenis",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "Regelt het aantal invoeren in de promptgeschiedenis die het systeem bewaart voor referentie.\n"
        "Een waarde van nul laat een onbeperkt aantal invoeren toe."
}



# Korean lexemes ai_assistant.py
lexemes = {
    # AI 보조
    "dialog_title": "AI에게 요청",
    "dialog_prompt_input_placeholder_text": "요청",
    "dialog_prompt_input_accessible_description": "AI 보조에게 컨텍스트를 제공하기 위한 요청 입력",
    "dialog_response_output_placeholder_text": "응답 데이터",
    "dialog_response_output_accessible_description": "응답 데이터가 이 필드에 표시됩니다",
    "dialog_response_output_notice_empty_text": "텍스트 입력을 제공하여 대략...",

    "dialog_usage_module_label": "모듈",
    "dialog_usage_model_label": "모델",
    "dialog_usage_tokens_label": "토큰",
    "dialog_usage_tokens_prompt": "프롬프트: {tokens}",
    "dialog_usage_tokens_answer": "답변: {tokens}",
    "dialog_usage_tokens_total": "총계: {tokens}",

    "dialog_button_send_request": "요청 보내기",
    "dialog_button_stop_request": "중지",
    "dialog_button_save_history": "대화 기록 저장",

    "dialog_message_copy_tooltip": "텍스트 복사",
    "dialog_message_copied_tooltip": "복사됨",

    "dialog_error_loading_model": "사용자 정의 모델 로딩 오류",
}



# Turkish lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "Cihazda LLM",

    "module_ondevice_llm_model_exception": "Model bulunamadı hatası: {error_msg}",
    "module_ondevice_llm_task_exception": "Model üreteci başlatılamıyor: {error_msg}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtWidgets import QMainWindow
from PySide6.QtGui import QTextDocument, QFontMetrics

from notolog.settings import Settings
from notolog.edit_widget import EditWidget
from notolog.enums.languages import Languages
from notolog.ui.line_numbers import LineNumbers

from . import test_app  # noqa: F401

from unittest.mock import MagicMock

import pytest


class TestLineNumbers:

    @pytest.fixture(scope="class", autouse=True)
    def settings_obj(self):
        """
        Use 'autouse=True' to enable automatic setup, or pass 'settings_obj' directly to main_window()
        """
        # Fixture to create and return settings instance
        settings = Settings()
        # Clear settings to be sure start over without side effects
        settings.clear()
        # Reset singleton (qa functionality)
        Settings.reload()

        # Remember to pass the fixture to the other fixtures (check ui_obj())
        yield settings

    @pytest.fixture
    def main_window(self, mocker, test_app):  # noqa: F811 redefinition of unused 'test_app'
        # Force to override system language as a default
        mocker.patch.object(Languages, 'default', return_value='la')

        # Fixture to create and return main window instance
        yield QMainWindow()

    @pytest.fixture(autouse=True)
    def ui_obj(self, mocker, settings_obj, main_window):

        mock_text_document = MagicMock(spec=QTextDocument)
        mocker.patch.object(mock_text_document, 'defaultFont', return_value='smth1')

        mocker.patch.object(EditWidget, 'document', return_value=mock_text_document)
        mocker.patch.object(EditWidget, 'parent', return_value=main_window)
        mock_edit_widget = EditWidget()

        # Fixture to initialize object.
        yield LineNumbers(editor=mock_edit_widget)

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        """
        Fixture to pass params and get expected results.
        This method is used only for passing params via pytest fixture.
        """
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    def test_ui_object(self, mocker, ui_obj: LineNumbers, settings_obj):
        # Check app language set correctly
        assert settings_obj.app_language == 'la'

        mock_update_numbers = mocker.patch.object(ui_obj, 'update_numbers', return_value=None)
        mock_update_request = mocker.patch.object(ui_obj, 'update_request', return_value=None)
        mock_update_width = mocker.patch.object(ui_obj, 'update_width', return_value=None)

        ui_obj.editor.setPlainText('smth1')

        mock_update_request.assert_called()
        assert 'QRect' in str(mock_update_request.call_args)
        mock_update_width.assert_not_called()

        ui_obj.editor.setPlainText('smth1\nsmth2\nsmth3')
        mock_update_width.assert_called_once()
        assert str(mock_update_width.call_args) == 'call()'

        assert ui_obj.line_numbers_width() > 0

        # In test mode it's only callable from the init method;
        # otherwise on block count changed from the outside
        mock_update_numbers.assert_not_called()

    @pytest.mark.parametrize(
        "test_exp_params_fixture",
        [
            (0, 0, 5),  # Digits: max(2, block_count)
            (1, 10, 25),  # Digits: max(2, block_count)
            (10, 10, 25),  # Digits: max(2, block_count)
            (100, 10, 35),
            (1000, 10, 45),
            (10000, 10, 55),
            (10000, 11, 60),
        ],
        indirect=True
    )
    def test_line_numbers_width(self, mocker, ui_obj: LineNumbers, settings_obj, test_exp_params_fixture):
        block_count, horizontal_advance, exp_result = test_exp_params_fixture

        mock_font_metrics = MagicMock(spec=QFontMetrics)
        mocker.patch.object(mock_font_metrics, 'horizontalAdvance', return_value=horizontal_advance)

        mocker.patch.object(ui_obj.editor, 'blockCount', return_value=block_count)
        mocker.patch.object(ui_obj, 'fontMetrics', return_value=mock_font_metrics)

        result = ui_obj.line_numbers_width()

        assert result == exp_result



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtGui import QTextDocument, QFont

from notolog.highlight.view_highlighter import ViewHighlighter
from notolog.view_processor import ViewProcessor
from notolog.view_decorator import ViewDecorator

import pytest


class TestHtmlView:

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_doc(self, request):
        """
        Either setHtml() or setMarkdown() is required as param_func
        * https://doc.qt.io/qt-6/qtextedit.html
        """

        if not hasattr(self, '_doc'):
            self._doc = QTextDocument()
            """ To avoid errors like: QFont::setPixelSize: Pixel size <= 0 (-1) """
            font = QFont("Sans Serif")
            self._doc.setDefaultFont(font)

        # Retrieve parameter values from the test request.
        param_value, param_func = request.param

        if callable(getattr(self._doc, param_func)):
            getattr(self._doc, param_func)(param_value)

        yield self._doc

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_highlighter(self, test_obj_doc):
        yield ViewHighlighter(document=test_obj_doc)

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_processor(self, test_obj_highlighter):
        yield ViewProcessor(highlighter=test_obj_highlighter)

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_decorator(self, test_obj_highlighter):
        yield ViewDecorator(highlighter=test_obj_highlighter)

    @pytest.fixture(scope="function")
    def test_text_fixture(self, request):
        # Get the parameter value from the request
        param_value = request.param

        yield param_value

    @pytest.fixture(scope="function")
    def test_style_fixture(self, request):
        # Get the parameter value from the request
        param_value = request.param

        yield param_value

    @pytest.mark.parametrize(
        "test_obj_doc, test_text_fixture, test_style_fixture",
        [
            (("<b>Test text bold</b>", "setHtml"), "text", "bold"),
            (("<i>Test text italic</i>", "setHtml"), "text", "italic"),
            (("<u>Test text underline</u>", "setHtml"), "text", "underline"),
            (("<s>Test text strikethrough</s>", "setHtml"), "text", "strikeOut"),
            # (("~~Test text strikethrough~~", "setHtml"), "text", "strikeOut"),  # TODO decorator result
            # Markdown is a proof of concept here as it is not in use for result rendering
            (("**Test text bold**", "setMarkdown"), "text", "bold"),
            (("*Test text bold*", "setMarkdown"), "text", "italic"),
            (("***Test text bold***", "setMarkdown"), "text", "bold"),
            (("***Test text bold***", "setMarkdown"), "text", "italic"),
            (("~~Test text strikethrough~~", "setMarkdown"), "text", "strikeOut"),
        ],
        indirect=True
    )
    def test_html_view_assertions(
            self, test_obj_doc, test_text_fixture, test_style_fixture, test_obj_processor, test_obj_decorator):
        """
        test_obj_doc.setHtml("<i>Test text italic</i>")
        test_obj_doc.setMarkdown('*Test text italic*')

        cursor = test_obj_doc.find(test_text_fixture)
        """

        """
        * https://doc.qt.io/qt-6/qtextdocument.html#find-1
        """
        cursor = test_obj_doc.find(test_text_fixture)

        _format = cursor.charFormat()
        assert cursor.selectedText()
        """
        test_style_fixture is a method name
        * https://doc.qt.io/qt-6/qfont.html#bold
        * https://doc.qt.io/qt-6/qfont.html#italic
        * https://doc.qt.io/qt-6/qfont.html#strikeOut
        * ...
        """
        assert callable(getattr(_format.font(), test_style_fixture))
        assert getattr(_format.font(), test_style_fixture)()

        """
        Assertion like ',-1,-1,5,700,0,0,0,0,0,0,0,0,0,0,1'
        * https://doc.qt.io/qt-6/qfont.html#toString
        """
        # assert not format.font().toString() == '...'



# Dutch lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "On Device LLM",

    "module_ondevice_llm_config_label": "On Device LLM Model",
    "module_ondevice_llm_config_path_label": "ONNX Model Locatie",
    "module_ondevice_llm_config_path_input_placeholder_text": "Pad naar de modeldirectory",
    "module_ondevice_llm_config_path_input_accessible_description":
        "Een invoerveld met een selector om het pad naar de modeldirectory te specificeren waar ONNX-bestanden zich\n"
        "bevinden. Ondersteunde modellen zijn in het ONNX-formaat, wat staat voor Open Neural Network Exchange,\n"
        "een open standaard formaat voor machineleermodellen.",

    "module_ondevice_llm_config_response_temperature_label": "Temperatuur: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "Past de willekeurigheid van de modelreacties aan. Hogere waarden produceren gevarieerdere outputs,\n"
        "terwijl lagere waarden reacties voorspelbaarder maken.",

    "module_ondevice_llm_config_response_max_tokens_label": "Maximale Respons Tokens",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "Stelt het maximale aantal tokens in dat in een reactie ontvangen kan worden, zoals woorden en interpunctie,\n"
        "en beheert de lengte van de output.",

    "module_ondevice_llm_config_prompt_history_size_label": "Grootte van de Promptgeschiedenis",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "Regelt het aantal invoeren in de promptgeschiedenis dat het systeem bewaart voor referentie.\n"
        "Een waarde van nul staat een onbeperkt aantal invoeren toe."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.encrypt.enc_helper import EncHelper
from notolog.encrypt.enc_password import EncPassword

from cryptography.fernet import Fernet

from logging import Logger

import pytest


class TestEncHelper:

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_enc_password(self, request):

        # Retrieve parameter values from the test request.
        password = request.param

        _enc_password = EncPassword()
        _enc_password.password = password
        _enc_password.hint = None

        yield _enc_password

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_enc_helper(self, mocker, test_obj_enc_password, request):
        # Retrieve parameter values from the test request.
        salt, iterations = request.param

        # Force fallback default to avoid settings default
        mocker.patch.object(EncHelper, 'get_default_iterations', return_value=EncHelper.DEFAULT_ITERATIONS)

        mocker.patch('secrets.token_urlsafe', return_value='RANDOM_SALT')
        _helper = EncHelper(enc_password=test_obj_enc_password, salt=salt, iterations=iterations)

        yield _helper

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        password, salt, iterations = request.param

        yield password, salt, iterations

    @pytest.mark.parametrize(
        "test_obj_enc_password, test_obj_enc_helper, test_exp_params_fixture",
        [
            ((None), (None, None), (b'', b'RANDOM_SALT', EncHelper.DEFAULT_ITERATIONS)),
            (('some>pass>here'), ('some>salt>here', 8000), (b'some>pass>here', b'some>salt>here', 8000)),
            ((b'some>pass>here'), (b'some>salt>here', 16000), (b'some>pass>here', b'some>salt>here', 16000)),
        ],
        indirect=True
    )
    def test_enc_helper_init(self, test_obj_enc_password, test_obj_enc_helper, test_exp_params_fixture):
        # Check initial params are exist
        assert isinstance(test_obj_enc_helper.logger, Logger)

        exp_password, exp_salt, exp_iterations = test_exp_params_fixture

        assert hasattr(test_obj_enc_helper, 'enc_password')
        assert hasattr(test_obj_enc_helper, 'password')
        assert hasattr(test_obj_enc_helper, 'salt')
        assert hasattr(test_obj_enc_helper, 'iterations')
        # Password and salt are expectedly set
        assert test_obj_enc_helper.enc_password == test_obj_enc_password
        assert test_obj_enc_helper.password == exp_password
        assert test_obj_enc_helper.salt == exp_salt
        # Check expected iterations count
        assert test_obj_enc_helper.iterations == exp_iterations

        assert hasattr(test_obj_enc_helper, 'cipher_suite')
        assert isinstance(test_obj_enc_helper.cipher_suite, Fernet)

        assert hasattr(test_obj_enc_helper, 'key')
        assert test_obj_enc_helper.key is None

    @pytest.fixture(scope="function")
    def test_exp_param_fixture(self, request):
        # Get the parameter value from the request
        param_value = request.param

        yield param_value

    @pytest.mark.parametrize(
        "test_obj_enc_password, test_obj_enc_helper, test_exp_param_fixture",
        [
            ((None), (None, 1024), False),
            ((''), (None, 1024), False),
            ((b''), (None, 1024), False),
            (('some>pass>here'), (None, 1024), True),
            ((b'some>pass>here'), (None, 1024), True),
        ],
        indirect=True
    )
    def test_enc_helper_is_password_valid(self, test_obj_enc_helper, test_exp_param_fixture):
        assert test_obj_enc_helper.is_password_valid() == test_exp_param_fixture

    @pytest.mark.parametrize(
        "test_obj_enc_password, test_obj_enc_helper, test_exp_param_fixture",
        [
            ((None), (None, 1024), 11),  # 'RANDOM_SALT' len is 11, mocked above. Usual len is 43
        ],
        indirect=True
    )
    def test_enc_helper_generate_salt(self, test_obj_enc_helper, test_exp_param_fixture):
        _salt = EncHelper.generate_salt()
        assert len(_salt) == test_exp_param_fixture
        assert type(_salt) is str

    @pytest.mark.parametrize(
        "test_obj_enc_password, test_obj_enc_helper, test_exp_param_fixture",
        [
            ((None), (None, 16000),
             b'-TK3+?nJ\xe49\x08\x13\x13_\x1f\xd0\xc5\xa2\x89ZZ,\x0f\x99\xfe\xd9\xe2\xa3'
             b'j\xe8\xee\xc3'),  # 'RANDOM_SALT' mocked above
            ((b'some>pass>here'), (b'some>salt>here', 16000),
             b'#\xb4\xebj\x9a\xe3Zg\x1a=v\x1a\x17\x02\xeb\x95\x88B(Uv]E\xb2y\x91=\xbeF\x84Bl'),
        ],
        indirect=True
    )
    def test_enc_helper_generate_key(self, test_obj_enc_helper, test_exp_param_fixture):

        assert test_obj_enc_helper.generate_key_from_password() == test_exp_param_fixture

    @pytest.fixture(scope="function")
    def test_content_fixture(self, request):
        # Get the parameter value from the request
        content = request.param

        yield content

    @pytest.fixture(scope="function")
    def test_exp_content_fixture(self, request):
        # Get the parameter value from the request
        content = request.param

        yield content

    @pytest.mark.parametrize(
        "test_obj_enc_password, test_obj_enc_helper, test_content_fixture, test_exp_content_fixture",
        [
            ((b'some>pass>here'), (b'some>salt>here', 16000),
             'Lorem ipsum dolor sit amet'.encode("utf-8"),
             b'Lorem ipsum dolor sit amet'),
        ],
        indirect=True
    )
    def test_enc_helper_encrypt_decrypt(self, test_obj_enc_helper, test_content_fixture, test_exp_content_fixture):

        _encrypted_content = test_obj_enc_helper.encrypt_data(test_content_fixture)

        assert len(_encrypted_content) > 0
        assert type(_encrypted_content) is bytes

        _decrypted_content = test_obj_enc_helper.decrypt_data(_encrypted_content)

        assert len(_decrypted_content) > 0
        assert type(_decrypted_content) is bytes
        assert _decrypted_content == test_content_fixture



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Tooltips helper class.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QPoint
from PySide6.QtWidgets import QWidget, QToolTip


class TooltipHelper(QWidget):

    @staticmethod
    def show_tooltip(widget: QWidget, text: str, offset=QPoint(20, -20)) -> None:
        """
        Displays a tooltip for a specified widget.

        This static method is used to show a tooltip near a widget with custom text and a positional offset.

        Args:
            widget (QWidget): The widget for which the tooltip is displayed.
            text (str): The text displayed inside the tooltip.
            offset (QPoint): The offset from the widget's position where the tooltip should be shown.

        Returns:
            None
        """

        """
        # Using the static method to show the tooltip
        TooltipManager.show_tooltip(self.button, "Copied!", QPoint(20, -20)
        """

        # Adjust where the tooltip should appear
        global_pos = widget.mapToGlobal(QPoint(0, 0)) + offset

        # Show tooltip
        QToolTip.showText(global_pos, text, widget)



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Extends QFileSystemModel functionality to provide data for file tree UI.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt
from PySide6.QtGui import QColor
from PySide6.QtWidgets import QFileSystemModel

import os

from typing import Any

from . import ThemeHelper

from ..helpers.file_helper import remove_trailing_numbers


class FileSystemModel(QFileSystemModel):
    def __init__(self, *args, **kwargs):
        super(FileSystemModel, self).__init__(*args, **kwargs)

        self.condition = None
        self.color = None

        # Theme helper
        self.theme_helper = ThemeHelper()

        self.highlighted_indexes = []

    def highlight(self, index, condition, color):
        """
        Keep in mind: the method has invoked on prev items atm.
        @param index: File index
        @param condition: File condition (name in this case)
        @param color: QColor for the role
        @return: None
        """
        self.condition = condition
        self.color = color

        if index.isValid():
            if not any(d.get('condition') == self.condition for d in self.highlighted_indexes):
                self.highlighted_indexes.append({'index': index, 'condition': condition})
                if len(self.highlighted_indexes) > 3:
                    # Or: self.highlighted_indexes = self.highlighted_indexes[-3:]
                    self.highlighted_indexes.pop(0)  # Store only the last X of them, remove the others
            # Data changed signal
            self.dataChanged.emit(index, index, [Qt.ItemDataRole.BackgroundRole])

    def clear_highlights(self):
        """
        Clear last X elements in the file tree. It may help when previous elements were highlighted special way.
        @return: None
        """
        for idx in self.highlighted_indexes:
            index = idx['index']
            condition = idx['condition']
            # Params the data() will be called with
            self.condition = condition
            # Allow to re-draw the role with default fallback color
            self.color = self.theme_helper.get_color('main_tree_background', True)
            # Emit the change
            self.dataChanged.emit(index, index, [Qt.ItemDataRole.BackgroundRole])
        self.highlighted_indexes.clear()

    def data(self, index=int, role=Qt.ItemDataRole.DisplayRole) -> Any:
        """
        Items view decoration.
        More info about ItemDataRole https://doc.qt.io/qt-6/qt.html#ItemDataRole-enum
        """

        # Item's background decoration
        if self.condition and role == Qt.ItemDataRole.BackgroundRole:
            text = index.data(Qt.ItemDataRole.DisplayRole)
            """
            Search 'like' match:
            if self.condition in text:
            """
            # Search exact match
            if self.condition == text:
                return QColor(self.color)

        # Item's text color decoration
        if role == Qt.ItemDataRole.ForegroundRole:
            file_path = self.filePath(index)
            if remove_trailing_numbers(file_path).endswith('del'):
                color = self.theme_helper.get_color('main_tree_file_type_del')
                return QColor(color)

        # Item's icon decoration
        if role == Qt.ItemDataRole.DecorationRole:
            info = self.fileInfo(index)
            """
            Decorate file's tree icon by file extension
            """
            # TODO more variants
            if os.path.isfile(info.filePath()):
                """
                Match case statement works for Python 3.10 upwards
                match info.suffix():
                    case 'md':
                        return ...
                Most common switch case
                """
                if info.suffix() == 'md':
                    color = self.theme_helper.get_color('main_tree_file_type_md')
                    return self.theme_helper.get_icon(theme_icon='filetype-md.svg', color=QColor(color))
                elif info.suffix() == 'txt':
                    color = self.theme_helper.get_color('main_tree_file_type_txt')
                    return self.theme_helper.get_icon(theme_icon='filetype-txt.svg', color=QColor(color))
                elif info.suffix() == 'html':
                    color = self.theme_helper.get_color('main_tree_file_type_html')
                    return self.theme_helper.get_icon(theme_icon='filetype-html.svg', color=QColor(color))
                elif info.suffix() == 'enc':
                    color = self.theme_helper.get_color('main_tree_file_type_enc')
                    return self.theme_helper.get_icon(theme_icon='file-earmark-lock2.svg', color=QColor(color))
                elif remove_trailing_numbers(info.suffix()) == 'del':
                    color = self.theme_helper.get_color('main_tree_file_type_del')
                    return self.theme_helper.get_icon(theme_icon='file-earmark-x.svg', color=QColor(color))
                else:
                    color = self.theme_helper.get_color('main_tree_file_type_default')
                    return self.theme_helper.get_icon(theme_icon='file-earmark.svg', color=QColor(color))
            elif os.path.isdir(info.filePath()):
                if info.fileName() == "..":
                    color = self.theme_helper.get_color('main_tree_folder_dotdot')
                    return self.theme_helper.get_icon(theme_icon='folder-symlink.svg', color=QColor(color))
                else:
                    color = self.theme_helper.get_color('main_tree_folder')
                    return self.theme_helper.get_icon(theme_icon='folder.svg', color=QColor(color))

        return super(FileSystemModel, self).data(index, role)



# Turkish lexemes common.py
lexemes = {
    "app_title": "Notolog Editörü",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "Dosya filtre alanı",

    "menu_action_copy_file_path": "Dosya yolunu kopyala",
    "menu_action_rename": "Yeniden Adlandır",
    "menu_action_delete": "Sil",
    "menu_action_delete_completely": "Tamamen Sil",
    "menu_action_restore": "Geri Yükle",
    "menu_action_create_new_dir": "Yeni dizin oluştur",

    "dialog_file_rename_title": "Dosyayı Yeniden Adlandır",
    "dialog_file_rename_field_label": "Yeni dosya adını girin",
    "dialog_file_rename_button_ok": "Yeniden Adlandır",
    "dialog_file_rename_warning_exists": "Aynı isimde dosya zaten var",

    "dialog_file_delete_title": "Dosyayı Sil",
    "dialog_file_delete_text": "\"{file_name}\" dosyasını sil?",
    "dialog_file_delete_completely_title": "Dosyayı Tamamen Sil",
    "dialog_file_delete_completely_text": "\"{file_name}\" dosyasını tamamen sil?",
    "dialog_file_delete_error": "Dosya silinemedi, hata oluştu",
    "dialog_file_delete_error_not_found": "Dosya bulunamadı",

    "dialog_file_restore_title": "Dosyayı Geri Yükle",
    "dialog_file_restore_text": "\"{file_name}\" dosyasını geri yükle?",
    "dialog_file_restore_error": "Dosya geri yüklenemedi, hata oluştu",
    "dialog_file_restore_warning_exists": "Aynı isimde dosya zaten var",

    "dialog_create_new_dir_title": "Yeni dizin oluştur",
    "dialog_create_new_dir_label": "Yeni dizin adı",
    "dialog_create_new_dir_input_placeholder_text": "Dizin adını girin",
    "dialog_create_new_dir_button_ok": "Oluştur",
    "dialog_create_new_dir_button_cancel": "İptal",
    "dialog_create_new_dir_warning_empty_title": "Yeni dizin adı hatası",
    "dialog_create_new_dir_warning_empty_text": "Dizin adı boş olamaz",
    "dialog_create_new_dir_warning_too_long_title": "Yeni dizin adı hatası",
    "dialog_create_new_dir_warning_too_long_text": "Dizin adı çok uzun; en fazla {symbols} karakter izin verilir!",
    "dialog_create_new_dir_error_existed": "Dizin zaten var",
    "dialog_create_new_dir_error": "Dizin oluşturulamıyor. Hedef dizinin {base_dir} yazılabilir olduğundan emin olun",

    "dialog_message_box_title": "Mesaj",
    "dialog_message_box_button_ok": "Kapat",

    "action_new_file_first_line_template_text": "Yeni belge",
    "action_open_file_dialog_caption": "Dosya Aç",
    "action_save_as_file_dialog_caption": "Dosyayı Farklı Kaydet",

    "dialog_save_empty_file_title": "Boş Dosyayı Kaydet",
    "dialog_save_empty_file_text": "Boş içerikli dosyayı kaydetmeye izin ver?",

    "dialog_encrypt_file_title": "Dosyayı Şifrele",
    "dialog_encrypt_file_text": "\"{file_name}\" dosyasını şifrele?",
    "encrypt_file_warning_file_is_already_encrypted": "Dosya zaten şifrelenmiş!",
    "dialog_encrypt_file_rewrite_existing_title": "Mevcut dosyayı yeniden yaz",
    "dialog_encrypt_file_rewrite_existing_text": "\"{file_path}\" mevcut dosyayı yeniden yaz?",

    "dialog_decrypt_file_title": "Dosyanın Şifresini Çöz",
    "dialog_decrypt_file_text": "\"{file_name}\" dosyasının şifresini çöz?",
    "decrypt_file_warning_file_is_not_encrypted": "Dosya şifreli değil!",
    "dialog_decrypt_file_rewrite_existing_title": "Mevcut dosyayı yeniden yaz",
    "dialog_decrypt_file_rewrite_existing_text": "\"{file_path}\" mevcut dosyayı yeniden yaz?",

    "dialog_encrypt_new_password_title": "Yeni Şifre",
    "dialog_encrypt_new_password_label": "Şifre:",
    "dialog_encrypt_new_password_input_placeholder_text": "Yeni Şifre Girin",
    "dialog_encrypt_new_password_hint_label": "İpucu:",
    "dialog_encrypt_new_password_hint_label_description":
        "İpucu şifrelenmez ve dosyadan okunabilir!"
        "\nKolay tahmin edilebilecek açık ipuçları kullanmayın, örneğin doğum tarihleri gibi."
        "\nKolayca ilişkilendirilemeyecek bir referans kullanın.",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "İpucu Girin (İsteğe Bağlı)",
    "dialog_encrypt_new_password_button_ok": "Tamam",
    "dialog_encrypt_new_password_button_cancel": "İptal",
    "dialog_encrypt_new_password_warning_empty_title": "Uyarı",
    "dialog_encrypt_new_password_warning_empty_text": "Şifre alanı boş olamaz!",
    "dialog_encrypt_new_password_warning_too_long_title": "Uyarı",
    "dialog_encrypt_new_password_warning_too_long_text": "İpucu alanı çok uzun, maksimum {symbols} karakter!",

    "dialog_encrypt_password_title": "Şifre Girin",
    "dialog_encrypt_password_label": "Şifre:",
    "dialog_encrypt_password_input_placeholder_text": "Şifre Girin",
    "dialog_encrypt_password_hint_label": "İpucu:",
    "dialog_encrypt_password_button_ok": "Tamam",
    "dialog_encrypt_password_button_cancel": "İptal",

    "dialog_encrypt_password_reset_title": "Şifreleme Şifresini Sıfırla",
    "dialog_encrypt_password_reset_text": "Mevcut şifreleme şifresini sıfırlamak istediğinizden emin misiniz?",
    "dialog_encrypt_password_reset_button_cancel": "İptal",
    "dialog_encrypt_password_reset_button_yes": "Evet",

    "dialog_open_link_title": "Bağlantı",
    "dialog_open_link_text": "\"{url}\" bağlantısını tarayıcıda aç?",

    "dialog_reset_settings_title": "Ayarları sıfırla?",
    "dialog_reset_settings_text":
        "Ayarlar içinde saklanan tüm veriler silinecek ve uygulama değişiklikleri uygulamak için yeniden başlatılacak.",

    "dialog_exit_unsaved_title": "Çıkışı Onayla",
    "dialog_exit_unsaved_text": "Açık olan dosya '{file_name}' kaydedilemiyor. Çıkışa devam edilsin mi?",

    "message_app_config_file_access": "{file_path} konumundaki uygulama yapılandırma dosyasına erişilirken izin reddedildi. "
                                      "Uygun işleyişi sağlamak için doğru izinleri ayarlayın.",

    "field_dir_path_dialog_caption": "Dizin Seç",
    "field_file_path_dialog_caption": "Dosya Seç",

    "load_file_encryption_password_mismatch": "Şifreleme şifresi uyuşmuyor!",
    "load_file_encryption_password_incorrect": "Şifreleme şifresi yanlış!",
    "load_file_none_content_error": "Dosya yüklenemedi.",

    "action_new_file_error_occurred": "Dosya oluşturulamıyor; bir hata oluştu.\nDosya sistemi izinlerini kontrol edin.",
    "save_active_file_error_occurred": "Dosya kaydedilemiyor; bir hata oluştu.",

    "expandable_block_default_title": "Daha fazla bilgi...",
    "expandable_block_open_close_tags_mismatch_warning": "<details> blok açma/kapama etiketleri uyuşmazlığı",

    "dialog_color_picker_color_copied_to_the_clipboard": "Biçimlendirilmiş metin panoya kopyalandı",

    "popup_about_title": "Uygulama Bilgisi",
    "popup_about_app_name_description": "Python Markdown Editörü",

    "popup_about_version": "Versiyon",
    "popup_about_license": "Lisans",
    "popup_about_website": "Web Sitesi",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "Tarih",

    "update_helper_new_version_is_available": "Uygulamanın yeni versiyonu {latest_version} mevcut",
    "update_helper_latest_version_installed": "Uygulamanın en güncel versiyonu yüklü",

    "network_connection_error_empty": "Yanıt bilgisi alınamıyor",
    "network_connection_error_connection_or_dns":
        "Sunucu bulunamadı. İnternet bağlantısında veya DNS'de bir sorun olabilir.",
    "network_connection_error_connection_refused":
        "Bağlantı reddedildi. Sunucu çökmüş olabilir veya ağ sorunları olabilir.",
    "network_connection_error_connection_timed_out": "Bağlantı zaman aşımına uğradı. Ağ sorunları olabilir.",
    "network_connection_error_connection_404_error":
        "404 bağlantı hatası. İstenen sayfa veya kaynak bulunamadı.",
    "network_connection_error_generic_with_status_code": "İstek, {status_code} durum kodu ile başarısız oldu.",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QTimer
from PySide6.QtGui import QTextDocument

from notolog.notolog_editor import NotologEditor
from notolog.settings import Settings
from notolog.editor_state import Mode
from notolog.lexemes.lexemes import Lexemes
from notolog.edit_widget import EditWidget
from notolog.file_header import FileHeader
from notolog.encrypt.enc_helper import EncHelper
from notolog.editor_state import Encryption

from logging import Logger

from unittest.mock import Mock, MagicMock

from types import SimpleNamespace

import pytest

import os


class TestNotologEditor:

    @pytest.fixture(scope="function")
    def test_obj_lexemes(self, mocker):
        # Return empty lexemes
        mocker.patch.object(Lexemes, 'load_lexemes', return_value={})
        # Init lexemes
        _lexemes = Lexemes()
        # Mock get lexeme results
        mocker.patch.object(_lexemes, 'get', return_value='Lorem ipsum')

        yield _lexemes

    @pytest.fixture(scope="function")
    def test_obj_notolog_editor(self, mocker, test_obj_lexemes):
        # Mock __init__ method
        mocker.patch.object(NotologEditor, '__init__', return_value=None)
        # Prevent resource processing
        mocker.patch.object(NotologEditor, 'process_document_images', return_value=None)
        # Activate quiet mode
        mocker.patch.object(NotologEditor, 'is_quiet_mode', return_value=True)

        _obj = NotologEditor()
        # Set lexemes object
        setattr(_obj, 'lexemes', test_obj_lexemes)

        _logger = MagicMock(spec=Logger)
        # Set logger
        setattr(_obj, 'logger', _logger)

        # Create nested mocks for toolbar.search_form
        search_form_mock = Mock()
        toolbar_mock = Mock(search_form=search_form_mock)

        # Assign the mock toolbar to the editor object
        _obj.toolbar = toolbar_mock

        yield _obj

    @pytest.fixture(scope="function")
    def test_obj_settings(self):
        yield Settings()

    def test_notolog_editor_load_default_page(self, mocker, test_obj_notolog_editor, test_obj_settings, tmp_path):
        """
        Check that the default page is loaded
        @param mocker: pytest fixture
        @param test_obj_notolog_editor: NotologEditor instance
        @param test_obj_settings: Settings instance
        @param tmp_path: fixture to get the path to a temporary directory
        @return: None
        """
        test_obj_notolog_editor.settings = test_obj_settings
        assert isinstance(test_obj_notolog_editor.settings, Settings)

        # Specify a side effect method for the mocked object
        mocker.patch('os.path.isfile', side_effect=lambda filename: filename.endswith('.md'))

        mock_method1 = mocker.patch.object(test_obj_notolog_editor, 'get_any_file', return_value=None)
        mock_method2 = mocker.patch.object(test_obj_notolog_editor, 'load_file', return_value=True)

        # Call the method under test
        result = test_obj_notolog_editor.load_default_page()
        assert result is True

        # Current working directory
        # test_run_from_dir = os.getcwd()
        test_file_dir = os.path.dirname(os.path.realpath(__file__))
        test_file_parent_dir = os.path.dirname(test_file_dir)
        # Assert that the method was called once
        mock_method1.assert_not_called()
        # Assert that the method was called once with the param(s)
        mock_method2.assert_called_once_with(os.path.normpath('%s/README.md') % test_file_parent_dir)

    def test_notolog_editor_load_default_page_any(self, mocker, test_obj_notolog_editor, test_obj_settings):
        """
        Check that an any page is loaded instead of default page
        @param mocker: pytest fixture
        @param test_obj_notolog_editor: NotologEditor instance
        @param test_obj_settings: Settings instance
        @return: None
        """
        # test_obj_notolog_editor.settings = mocker.Mock()
        test_obj_notolog_editor.settings = test_obj_settings
        assert isinstance(test_obj_notolog_editor.settings, Settings)

        mocker.patch.object(os.path, 'isfile', return_value=False)

        mock_method1 = mocker.patch.object(test_obj_notolog_editor, 'get_any_file', return_value='Test_File_Path.Py')
        mock_method2 = mocker.patch.object(test_obj_notolog_editor, 'load_file', return_value=True)

        # Call the method under test
        result = test_obj_notolog_editor.load_default_page()
        assert result is True

        # Assert that the method was called once
        mock_method1.assert_called_once()
        # Assert that the method was called once with the param(s)
        mock_method2.assert_called_once_with('Test_File_Path.Py')

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.mark.parametrize(
        "test_exp_params_fixture",
        [
            (Mode.VIEW, 'new-document-1.md', None, None, 1, True, True),
            (Mode.VIEW, 'new-document-1.md', None, None, 1, False, False),
            (Mode.VIEW, 'new-document-1.md', False, None, 1, True, True),
            (Mode.VIEW, 'new-document-9999.md', True, None, 9999, True, False),
            (Mode.VIEW, 'new-document-1.md', False, 'Lorem ipsum', 1, True, True),
        ],
        indirect=True
    )
    def test_notolog_editor_action_new_file(self, mocker, test_obj_notolog_editor, test_exp_params_fixture):
        mode, file_path, isfile, content, res_path_call_cnt, res_save_file, res_exp = test_exp_params_fixture

        mocker.patch.object(test_obj_notolog_editor, 'get_mode', return_value=mode)
        mocker.patch.object(test_obj_notolog_editor, 'toggle_mode', return_value=None)
        setattr(test_obj_notolog_editor, 'debug', False)
        setattr(test_obj_notolog_editor, 'logging', False)

        mock_res_path = mocker.patch.object(test_obj_notolog_editor, 'get_tree_active_dir', return_value=file_path)
        # mock_res_path = mocker.patch('app.notolog_editor.res_path', return_value=file_path)
        # fixture arg: monkeypatch
        # monkeypatch.setattr('app.notolog_editor.res_path', lambda _file_path: _file_path == file_path)
        mocker.patch.object(os.path, 'isfile', return_value=isfile)

        mock_save_file_content = mocker.patch.object(test_obj_notolog_editor, 'save_file_content', return_value=res_save_file)
        mocker.patch.object(test_obj_notolog_editor, 'load_file', return_value=True)

        result = test_obj_notolog_editor.action_new_file(content)

        assert result == res_exp
        assert mock_res_path.call_count == res_path_call_cnt
        # Check the parameters passed to the mocked function(s)
        # assert str(mock_res_path.call_args) == "call('%s')" % file_path
        # assert str(mock_save_file_content.call_args) == "call('%s', '%s')" % (file_path, content)
        if content:
            assert str(content) in str(mock_save_file_content.call_args)

    @pytest.mark.parametrize(
        "test_exp_params_fixture",
        [
            # mode, encryption, file_path, file_exists, file_write_ok, dir_write_ok
            (None, None, 'some-doc-1.md', None, None, None,
             # content, prev_content, clear_after, allow_save_empty_content, res_save_file, res_exp
             '', None, None, None, None, None),
            # Verify behavior when running in an unexpected mode
            (Mode.VIEW, Encryption.PLAIN, 'some-doc-1.md', False, False, False,
             '', None, False, None, False, None),
            (Mode.VIEW, Encryption.PLAIN, 'some-doc-1.md', False, False, False,
             'Lorem ipsum', 'To be or not to be?', False, None, False, None),
            # Manage scenarios involving saving empty content to the file
            (Mode.EDIT, Encryption.PLAIN, 'some-doc-1.md', None, None, None,
             '', None, False, None, False, False),
            (Mode.EDIT, Encryption.PLAIN, 'some-doc-1.md', True, True, True,
             '', 'Essentiam vel non essentiam? Hoc quaestio est.', False, None, False, None),
            (Mode.EDIT, Encryption.PLAIN, 'some-doc-1.md', True, True, True,
             '', 'Essentiam vel non essentiam? Hoc quaestio est.', False, True, False, False),
            (Mode.EDIT, Encryption.PLAIN, 'some-doc-1.md', True, True, True,
             '', 'Essentiam vel non essentiam? Hoc quaestio est.', False, True, True, True),
            # Insufficient permissions to save the file
            (Mode.EDIT, Encryption.PLAIN, 'some-doc-1.md', False, False, False,
             'Lorem ipsum', 'To be or not to be?', False, None, False, False),
            (Mode.EDIT, Encryption.PLAIN, 'some-doc-1.md', True, False, True,
             'Lorem ipsum', 'To be or not to be?', False, None, False, False),
            (Mode.EDIT, Encryption.ENCRYPTED, 'some-doc-1.md', False, False, False,
             'Lorem ipsum', 'To be or not to be?', False, None, False, False),
            # Handle other unexpected errors when saving the file
            (Mode.EDIT, Encryption.ENCRYPTED, 'some-doc-1.md', True, True, True,
             'Lorem ipsum', 'To be or not to be?', False, None, False, False),
            # Handle cases where the file can be saved
            (Mode.EDIT, Encryption.PLAIN, 'some-doc-1.md', False, True, True,
             'Lorem ipsum', 'To be or not to be?', False, None, True, True),
            (Mode.EDIT, Encryption.PLAIN, 'some-doc-1.md', True, True, False,
             'Lorem ipsum', 'To be or not to be?', False, None, True, True),
            (Mode.EDIT, Encryption.PLAIN, 'some-doc-1.md', True, True, True,
             'Lorem ipsum', 'To be or not to be?', False, None, True, True),
            (Mode.EDIT, Encryption.ENCRYPTED, 'some-doc-1.md', True, True, True,
             'Lorem ipsum', 'To be or not to be?', False, None, True, True),
            (Mode.EDIT, Encryption.ENCRYPTED, 'some-doc-1.md', True, True, True,
             'Lorem ipsum', 'To be or not to be?', True, None, True, True),
        ],
        indirect=True
    )
    def test_notolog_editor_save_active_file(self, mocker, test_obj_notolog_editor, test_exp_params_fixture):
        (mode, encryption, file_path, file_exists, file_write_ok, dir_write_ok, content, prev_content,
         clear_after, allow_save_empty_content, res_save_file, res_exp) \
            = test_exp_params_fixture

        mocker.patch.object(test_obj_notolog_editor, 'get_mode', return_value=mode)
        mocker.patch.object(test_obj_notolog_editor, 'toggle_mode', return_value=None)
        setattr(test_obj_notolog_editor, 'debug', False)
        setattr(test_obj_notolog_editor, 'logging', False)

        mocker.patch.object(test_obj_notolog_editor, 'get_current_file_path', return_value=file_path)

        mock_store_doc_cursor_pos = mocker.patch.object(test_obj_notolog_editor, 'store_doc_cursor_pos',
                                                        return_value=None)
        mock_toggle_save_timer = mocker.patch.object(test_obj_notolog_editor, 'toggle_save_timer', return_value=None)
        mock_message_box = mocker.patch.object(test_obj_notolog_editor, 'message_box', return_value=None)
        mock_common_dialog = mocker.patch.object(test_obj_notolog_editor, 'common_dialog', return_value=None)
        mock_get_encryption = mocker.patch.object(test_obj_notolog_editor, 'get_encryption', return_value=encryption)

        mock_edit_widget = MagicMock(spec=EditWidget)
        mock_edit_widget_to_plain_text = mocker.patch.object(mock_edit_widget, 'toPlainText', return_value=content)
        mock_edit_widget_clear = mocker.patch.object(mock_edit_widget, 'clear', return_value=None)
        mocker.patch.object(test_obj_notolog_editor, 'get_edit_widget', return_value=mock_edit_widget)

        setattr(test_obj_notolog_editor, 'content', prev_content)
        setattr(test_obj_notolog_editor, 'estate', SimpleNamespace(**{'allow_save_empty': allow_save_empty_content}))

        mock_save_timer_ui_icon = mocker.patch.object(QTimer, 'singleShot', return_value=None)

        # Handle file system permissions
        mocker.patch.object(os.path, 'exists', return_value=file_exists)
        write_permissions_matrix = {file_path: file_write_ok, os.path.dirname(file_path): dir_write_ok}
        mocker.patch.object(os, 'access',
                            side_effect=lambda key, _mode: getattr(SimpleNamespace(**write_permissions_matrix), key, None))

        # Lexemes
        mock_lexemes_get = mocker.patch.object(test_obj_notolog_editor.lexemes, 'get')

        # Process the file's header information
        mock_header = MagicMock(spec=FileHeader)
        mock_header_refresh = mocker.patch.object(mock_header, 'refresh', return_value=None)
        mock_header_pack = mocker.patch.object(mock_header, 'pack', wraps=mock_header.pack)
        header_line_tpl = '<!-- %s -->'  # To simplify the emulation of header processing operations
        mocker.patch.object(mock_header, '__repr__', return_value=(header_line_tpl % file_path))
        mocker.patch.object(mock_header, 'is_valid', return_value=True)
        mock_header_validate_enc = mocker.patch.object(mock_header, 'validate_enc', return_value=None)
        mocker.patch.object(mock_header, 'get_enc_param',
                            side_effect=lambda key: getattr(SimpleNamespace(**{'slt': 'any-salt', 'itr': 1}), key, None))
        setattr(test_obj_notolog_editor, 'header', mock_header)

        mock_enc_helper = MagicMock(spec=EncHelper)
        mock_enc_helper_encrypt_data = mocker.patch.object(mock_enc_helper, 'encrypt_data',
                                                           return_value=('ENCRYPTED' + content).encode('utf-8'))
        mock_get_encrypt_helper = mocker.patch.object(test_obj_notolog_editor, 'get_encrypt_helper',
                                                      return_value=mock_enc_helper)

        mock_save_file_content = mocker.patch.object(test_obj_notolog_editor, 'save_file_content',
                                                     return_value=res_save_file)

        # Save the active file
        result = test_obj_notolog_editor.save_active_file(clear_after, allow_save_empty_content)

        assert result == res_exp
        mock_store_doc_cursor_pos.assert_called_once()

        if res_save_file:
            mock_header_refresh.assert_called_once()
            mock_header_pack.assert_called_once()
            # Verify the parameters passed to the mocked function(s)
            # assert str(mock_res_path.call_args) == "call('%s')" % file_path
            # assert str(mock_save_file_content.call_args) == "call('%s', '%s')" % (file_path, content)
            assert str(content) in str(mock_header_pack.call_args)
            mock_edit_widget_to_plain_text.assert_called_once()
            assert mock_edit_widget_clear.call_count == int(clear_after)
            mock_get_encryption.assert_called_once()
            mock_toggle_save_timer.assert_not_called()
            mock_message_box.assert_not_called()
            assert test_obj_notolog_editor.content == content
            assert repr(test_obj_notolog_editor.header) == header_line_tpl % file_path
            # Encrypted file
            if encryption == Encryption.ENCRYPTED:
                mock_header_validate_enc.assert_called_once()
                mock_get_encrypt_helper.assert_called_once_with(salt='any-salt', iterations=1)
                mock_enc_helper_encrypt_data.assert_called_once_with(content.encode('utf-8'))
            else:
                mock_header_validate_enc.assert_not_called()
                mock_get_encrypt_helper.assert_not_called()
        elif mock_save_timer_ui_icon.call_count > 0:
            mock_message_box.assert_called_once()
            # Checks if the method was called with the parameter at least once
            mock_lexemes_get.assert_any_call('save_active_file_error_occurred')

        if (file_exists and file_write_ok) or (not file_exists and dir_write_ok):
            if not content:
                if allow_save_empty_content:
                    mock_save_timer_ui_icon.assert_called_once()
                else:
                    mock_save_timer_ui_icon.assert_not_called()
                    mock_common_dialog.assert_called_once()
                    # Checks if the method was called with the parameter at least once
                    mock_lexemes_get.assert_any_call('dialog_save_empty_file_title')
            elif mode == Mode.EDIT:
                mock_save_file_content.assert_called_once()
                mock_save_timer_ui_icon.assert_called_once()
                if res_save_file:
                    # No lexemes were retrieved for any dialog
                    mock_lexemes_get.assert_not_called()

        if not file_exists and not dir_write_ok and mode == Mode.EDIT:
            mock_toggle_save_timer.assert_called_once()
            mock_message_box.assert_called_once()
            # Checks if the method was called with the parameter at least once
            mock_lexemes_get.assert_any_call('save_active_file_error_occurred')

    @pytest.mark.parametrize(
        "test_exp_params_fixture",
        [
            (True, None, ''),
            (False, '', ''),
            (False, 'Test Text', 'Test Text'),
        ],
        indirect=True
    )
    def test_get_action_search_text(self, mocker, test_obj_notolog_editor, test_exp_params_fixture):
        param_default, param_text, text_result = test_exp_params_fixture

        setattr(test_obj_notolog_editor, 'debug', False)

        # Mock the search_form directly on the already provided fixture object
        search_form_mock = mocker.Mock()

        mocker.patch.object(test_obj_notolog_editor.toolbar, 'search_form', new=search_form_mock)
        # Set return values or side effects on this mock
        search_form_mock.text.return_value = param_text

        if param_default:
            # Force the method to use its default value by omitting the required parameter
            delattr(test_obj_notolog_editor.toolbar, 'search_form')

        result = test_obj_notolog_editor.get_action_search_text()

        assert result == text_result

        if not param_default:
            search_form_mock.text.assert_called_once()

    @pytest.mark.parametrize(
        "test_exp_params_fixture",
        [
            (None, None, QTextDocument.FindFlag(0)),
            (False, None, QTextDocument.FindFlag(0)),
            (True, None, QTextDocument.FindFlag.FindBackward),
            (False, False, QTextDocument.FindFlag(0)),
            (True, False, QTextDocument.FindFlag.FindBackward),
            (False, True, QTextDocument.FindFlag.FindCaseSensitively),
            (True, True, QTextDocument.FindFlag.FindBackward | QTextDocument.FindFlag.FindCaseSensitively),
        ],
        indirect=True
    )
    def test_get_action_search_flags(self, mocker, test_obj_notolog_editor, test_exp_params_fixture):
        param_backward, param_case_sensitive, flags_result = test_exp_params_fixture

        setattr(test_obj_notolog_editor, 'debug', False)

        # Mock the search_form directly on the already provided fixture object
        search_form_mock = mocker.Mock()
        mocker.patch.object(test_obj_notolog_editor.toolbar, 'search_form', new=search_form_mock)

        # Set return values or side effects on this mock
        search_form_mock.case_sensitive.return_value = param_case_sensitive

        result = test_obj_notolog_editor.get_action_search_flags(param_backward)

        assert result == flags_result
        search_form_mock.case_sensitive.assert_called_once()

    def test_is_quiet_mode(self, test_obj_notolog_editor):
        assert test_obj_notolog_editor.is_quiet_mode()



# Dutch lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "Kies een kleur",

    "color_picker_color_black": "Zwart",
    "color_picker_color_silver": "Zilver",
    "color_picker_color_gray": "Grijs",
    "color_picker_color_white": "Wit",
    "color_picker_color_maroon": "Kastanjebruin",
    "color_picker_color_red": "Rood",
    "color_picker_color_purple": "Paars",
    "color_picker_color_fuchsia": "Fuchsia",
    "color_picker_color_green": "Groen",
    "color_picker_color_lime": "Limoen",
    "color_picker_color_olive": "Olijf",
    "color_picker_color_yellow": "Geel",
    "color_picker_color_navy": "Marineblauw",
    "color_picker_color_blue": "Blauw",
    "color_picker_color_teal": "Blauwgroen",
    "color_picker_color_aqua": "Aqua",
    "color_picker_color_aliceblue": "Aliceblauw",
    "color_picker_color_antiquewhite": "Antiek wit",
    "color_picker_color_aquamarine": "Aquamarijn",
    "color_picker_color_azure": "Azuriet",
    "color_picker_color_beige": "Beige",
    "color_picker_color_bisque": "Bisque",
    "color_picker_color_blanchedalmond": "Geblancheerde amandel",
    "color_picker_color_blueviolet": "Blauwviolet",
    "color_picker_color_brown": "Bruin",
    "color_picker_color_burlywood": "Ruwe houtkleur",
    "color_picker_color_cadetblue": "Cadetblauw",
    "color_picker_color_chartreuse": "Chartreuse",
    "color_picker_color_chocolate": "Chocoladebruin",
    "color_picker_color_coral": "Koraal",
    "color_picker_color_cornflowerblue": "Korenbloemblauw",
    "color_picker_color_cornsilk": "Maïszijde",
    "color_picker_color_crimson": "Karmozijn",
    "color_picker_color_cyan": "Cyaan",
    "color_picker_color_darkblue": "Donkerblauw",
    "color_picker_color_darkcyan": "Donkercyaan",
    "color_picker_color_darkgoldenrod": "Donkergoud",
    "color_picker_color_darkgray": "Donkergrijs",
    "color_picker_color_darkgreen": "Donkergroen",
    "color_picker_color_darkkhaki": "Donkerkaki",
    "color_picker_color_darkmagenta": "Donkermagenta",
    "color_picker_color_darkolivegreen": "Donkerolijfgroen",
    "color_picker_color_darkorange": "Donkeroranje",
    "color_picker_color_darkorchid": "Donkerorchidee",
    "color_picker_color_darkred": "Donkerrood",
    "color_picker_color_darksalmon": "Donkerzalm",
    "color_picker_color_darkseagreen": "Donkerzeegroen",
    "color_picker_color_darkslateblue": "Donker leiblauw",
    "color_picker_color_darkslategray": "Donker leigrijs",
    "color_picker_color_darkturquoise": "Donkerturkoois",
    "color_picker_color_darkviolet": "Donkerviolet",
    "color_picker_color_deeppink": "Dieproze",
    "color_picker_color_deepskyblue": "Diephemelsblauw",
    "color_picker_color_dimgray": "Gedimd grijs",
    "color_picker_color_dodgerblue": "Dodgerblauw",
    "color_picker_color_firebrick": "Vuursteenrood",
    "color_picker_color_floralwhite": "Bloemwit",
    "color_picker_color_forestgreen": "Bosgroen",
    "color_picker_color_gainsboro": "Gainsboro",
    "color_picker_color_ghostwhite": "Spookwit",
    "color_picker_color_gold": "Goud",
    "color_picker_color_goldenrod": "Goudbruin",
    "color_picker_color_greenyellow": "Groengeel",
    "color_picker_color_honeydew": "Honingdauw",
    "color_picker_color_hotpink": "Felroze",
    "color_picker_color_indianred": "Indiaansrood",
    "color_picker_color_indigo": "Indigo",
    "color_picker_color_ivory": "Ivoor",
    "color_picker_color_khaki": "Kaki",
    "color_picker_color_lavender": "Lavendel",
    "color_picker_color_lavenderblush": "Lavendelroze",
    "color_picker_color_lawngreen": "Grasgroen",
    "color_picker_color_lemonchiffon": "Citroenchiffon",
    "color_picker_color_lightblue": "Lichtblauw",
    "color_picker_color_lightcoral": "Lichtkoraal",
    "color_picker_color_lightcyan": "Lichtcyaan",
    "color_picker_color_lightgoldenrodyellow": "Licht goudgeel",
    "color_picker_color_lightgray": "Lichtgrijs",
    "color_picker_color_lightgreen": "Lichtgroen",
    "color_picker_color_lightpink": "Lichtroze",
    "color_picker_color_lightsalmon": "Lichtzalm",
    "color_picker_color_lightseagreen": "Lichtzeegroen",
    "color_picker_color_lightskyblue": "Lichthemelsblauw",
    "color_picker_color_lightslategray": "Licht leigrijs",
    "color_picker_color_lightsteelblue": "Licht staalblauw",
    "color_picker_color_lightyellow": "Lichtgeel",
    "color_picker_color_limegreen": "Limetgroen",
    "color_picker_color_linen": "Linnen",
    "color_picker_color_magenta": "Magenta",
    "color_picker_color_mediumaquamarine": "Middelaquamarijn",
    "color_picker_color_mediumblue": "Middenblauw",
    "color_picker_color_mediumorchid": "Middenorchidee",
    "color_picker_color_mediumpurple": "Middenpaars",
    "color_picker_color_mediumseagreen": "Middenseegroen",
    "color_picker_color_mediumslateblue": "Midden leiblauw",
    "color_picker_color_mediumspringgreen": "Midden lentegroen",
    "color_picker_color_mediumturquoise": "Middenturkoois",
    "color_picker_color_mediumvioletred": "Middenvioletrood",
    "color_picker_color_midnightblue": "Middernachtblauw",
    "color_picker_color_mintcream": "Muntcrème",
    "color_picker_color_mistyrose": "Mistige roos",
    "color_picker_color_moccasin": "Moccasin",
    "color_picker_color_navajowhite": "Navajo wit",
    "color_picker_color_oldlace": "Oud kant",
    "color_picker_color_olivedrab": "Olijfgrijs",
    "color_picker_color_orange": "Oranje",
    "color_picker_color_orangered": "Oranjerood",
    "color_picker_color_orchid": "Orchidee",
    "color_picker_color_palegoldenrod": "Bleekgoud",
    "color_picker_color_palegreen": "Bleekgroen",
    "color_picker_color_paleturquoise": "Bleekturkoois",
    "color_picker_color_palevioletred": "Bleekvioletrood",
    "color_picker_color_papayawhip": "Papajakleur",
    "color_picker_color_peachpuff": "Perzik",
    "color_picker_color_peru": "Peru",
    "color_picker_color_pink": "Roze",
    "color_picker_color_plum": "Pruim",
    "color_picker_color_powderblue": "Poederblauw",
    "color_picker_color_rebeccapurple": "Rebecca paars",
    "color_picker_color_rosybrown": "Rozebruin",
    "color_picker_color_royalblue": "Koningsblauw",
    "color_picker_color_saddlebrown": "Zadelbruin",
    "color_picker_color_salmon": "Zalm",
    "color_picker_color_sandybrown": "Zandbruin",
    "color_picker_color_seagreen": "Zeegroen",
    "color_picker_color_seashell": "Schelp",
    "color_picker_color_sienna": "Sienna",
    "color_picker_color_skyblue": "Hemelsblauw",
    "color_picker_color_slateblue": "Leiblauw",
    "color_picker_color_slategray": "Leigrijs",
    "color_picker_color_snow": "Sneeuw",
    "color_picker_color_springgreen": "Lentegroen",
    "color_picker_color_steelblue": "Staalblauw",
    "color_picker_color_tan": "Bruinbeige",
    "color_picker_color_thistle": "Distel",
    "color_picker_color_tomato": "Tomaatrood",
    "color_picker_color_turquoise": "Turkoois",
    "color_picker_color_violet": "Violet",
    "color_picker_color_wheat": "Tarwe",
    "color_picker_color_whitesmoke": "Witte rook",
    "color_picker_color_yellowgreen": "Geelgroen"
}



# Portuguese lexemes toolbar.py
lexemes = {
    "actions_label_new_document": "Novo documento",
    "actions_accessible_name_new_document": "Novo documento",
    "actions_label_open": "Abrir",
    "actions_accessible_name_open": "Abrir documento",
    "actions_label_save": "Salvar",
    "actions_accessible_name_save": "Salvar documento",
    "actions_label_save_as": "Salvar como",
    "actions_accessible_name_save_as": "Salvar como cópia do documento",
    "actions_label_sync": "Sincronizar",
    "actions_accessible_name_sync": "Sincronizar documento com armazenamento externo",
    "actions_label_edit": "Editar",
    "actions_accessible_name_edit": "Editar arquivo",
    "actions_label_source": "Fonte",
    "actions_accessible_name_source": "Código-fonte",
    "actions_label_encrypt": "Criptografar",
    "actions_accessible_name_encrypt": "Criptografar arquivo",
    "actions_label_decrypt": "Descriptografar",
    "actions_accessible_name_decrypt": "Descriptografar arquivo",
    "actions_label_bold": "Negrito",
    "actions_accessible_name_bold": "Formatar texto em negrito",
    "actions_label_italic": "Itálico",
    "actions_accessible_name_italic": "Formatar texto em itálico",
    "actions_label_underline": "Sublinhado",
    "actions_accessible_name_underline": "Formatar texto com sublinhado",
    "actions_label_strikethrough": "Riscado",
    "actions_accessible_name_strikethrough": "Formatar texto como riscado",
    "actions_label_blockquote": "Citação",
    "actions_accessible_name_blockquote": "Formatar texto como citação",
    "actions_label_color_picker": "Seletor de cores",
    "actions_accessible_name_color_picker": "Escolher uma cor de texto",
    "actions_label_ai_assistant": "Assistente de IA",
    "actions_accessible_name_ai_assistant": "Executar assistente de IA",
    "actions_label_about": "Sobre",
    "actions_accessible_name_about": "Sobre o App",
    "actions_label_exit": "Sair",
    "actions_accessible_name_exit": "Sair do App",
    "actions_label_settings": "Configurações",
    "actions_accessible_name_settings": "Configurações do aplicativo",

    "search_input_label": "Pesquisar",
    "search_input_placeholder_text": "Pesquisar texto no documento",
    "search_input_accessible_description": "Campo de entrada de pesquisa",
    "search_buttons_label_clear": "Limpar pesquisa",
    "search_buttons_accessible_name_clear": "Limpar campo de pesquisa",
    "search_buttons_label_prev": "Pesquisa anterior",
    "search_buttons_accessible_name_prev": "Resultado da pesquisa anterior",
    "search_buttons_label_next": "Próxima pesquisa",
    "search_buttons_accessible_name_next": "Resultado da próxima pesquisa",
    "search_case_sensitive_accessible_name": "Diferenciar maiúsculas de minúsculas",
    "search_case_sensitive_tooltip": "Pesquisa diferenciando maiúsculas de minúsculas",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Base Enum class to support extension classes with minimal required functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from enum import Enum


class EnumBase(Enum):

    def __init__(self, value, is_default=False, *args, **kwargs):
        self._value_ = value
        self.is_default = is_default
        super().__init__(*args, **kwargs)

    def __str__(self):
        return self.name.lower()

    @classmethod
    def __getitem__(cls, key):
        if isinstance(key, str):
            key = key.upper()  # Normalize the key to uppercase
        try:
            return cls._member_map_[key]
        except KeyError:
            raise KeyError(f"{key} is not a valid key for {cls.__name__}")

    @classmethod
    def default(cls):
        """
        Returns the default value of the Enum.

         Returns:
            BaseEnum, None: Returns the enum member marked as default (is_default=True),
                            or None if no member is set as default.

        Example:
            BaseEnum.default()
        """
        # Find and return the default enum member
        for member in cls:
            if hasattr(member, 'is_default') and member.is_default:
                return member
        return None  # If no default is marked, return None or consider raising an error


def enum_factory(name, base_enum: EnumBase, **new_members):
    """
    Create a new enum or extend an existing enum with new members.

    Args:
        name (str): The name of the new enum class
        base_enum (EnumBase): Optional base enum to extend
        new_members: Additional or new members to include

    Example:
        ExtendedColorEnum = enum_factory('ExtendedColorEnum', ColorEnum, ORANGE='orange')
    """
    members = {item.name: item.value for item in base_enum}
    members.update(new_members)
    return Enum(name, members)



# Chinese lexemes toolbar.py
lexemes = {
    "actions_label_new_document": "新建文档",
    "actions_accessible_name_new_document": "新文档",
    "actions_label_open": "打开",
    "actions_accessible_name_open": "打开文档",
    "actions_label_save": "保存",
    "actions_accessible_name_save": "保存文档",
    "actions_label_save_as": "另存为",
    "actions_accessible_name_save_as": "另存文档副本",
    "actions_label_sync": "同步",
    "actions_accessible_name_sync": "与外部存储同步文档",
    "actions_label_edit": "编辑",
    "actions_accessible_name_edit": "编辑文件",
    "actions_label_source": "源码",
    "actions_accessible_name_source": "源代码",
    "actions_label_encrypt": "加密",
    "actions_accessible_name_encrypt": "加密文件",
    "actions_label_decrypt": "解密",
    "actions_accessible_name_decrypt": "解密文件",
    "actions_label_bold": "加粗",
    "actions_accessible_name_bold": "加粗文本格式",
    "actions_label_italic": "斜体",
    "actions_accessible_name_italic": "斜体文本格式",
    "actions_label_underline": "下划线",
    "actions_accessible_name_underline": "下划线文本格式",
    "actions_label_strikethrough": "删除线",
    "actions_accessible_name_strikethrough": "删除线文本格式",
    "actions_label_blockquote": "引用",
    "actions_accessible_name_blockquote": "引用文本格式",
    "actions_label_color_picker": "颜色选择器",
    "actions_accessible_name_color_picker": "选择文字颜色",
    "actions_label_ai_assistant": "AI 助手",
    "actions_accessible_name_ai_assistant": "运行 AI 助手",
    "actions_label_about": "关于",
    "actions_accessible_name_about": "关于应用",
    "actions_label_exit": "退出",
    "actions_accessible_name_exit": "退出应用",
    "actions_label_settings": "设置",
    "actions_accessible_name_settings": "应用设置",

    "search_input_label": "搜索",
    "search_input_placeholder_text": "在文档中搜索文本",
    "search_input_accessible_description": "搜索输入字段",
    "search_buttons_label_clear": "清除搜索",
    "search_buttons_accessible_name_clear": "清除搜索输入字段",
    "search_buttons_label_prev": "上一次搜索",
    "search_buttons_accessible_name_prev": "上一次搜索结果",
    "search_buttons_label_next": "下一次搜索",
    "search_buttons_accessible_name_next": "下一次搜索结果",
    "search_case_sensitive_accessible_name": "区分大小写",
    "search_case_sensitive_tooltip": "区分大小写的搜索",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Enum class with possible LLM chat formats, e.g. 'chatml', 'llama-2', 'gemma', etc..

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from .enum_base import EnumBase

assert hasattr(EnumBase, 'default'), "Check default() method is implemented in the base class"


class LlmChatFormats(EnumBase):

    # First param is format name
    # Second param in the tuple corresponds to default value
    CHATML = ("chatml", True)
    LLAMA2 = "llama-2"
    GEMMA = "gemma"
